# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
r"""
This code is borrowed and modified based on the source code from the
'MedCalc-Bench' repository.
Original repository: https://github.com/ncbi-nlp/MedCalc-Bench

Modifications include:
- rewrite function calculate_pe_wells_explanation
- translation

Date: March 2025
"""


def calculate_pe_wells_explanation(variables):
    r"""
    Calculates the patient's score of Wells' criteria for Pulmonary Embolism
    and generates a detailed explanatory text.

    Parameters:
        variables (dict): A dictionary containing the following
            key-value pairs:
            - "clinical_dvt" (boolean): Clinical signs and symptoms of DVT:
            No = 0 points, Yes = +3 points.
            - "previous_pe" (boolean): Previous, objectively diagnosed PE:
            No = 0 points, Yes = +1.5 points.
            - "previous_dvt" (boolean): Previous, objectively diagnosed DVT:
            No = 0 points, Yes = +1.5 points.
            - "heart_rate" (tuple): The patient's heart rate in the format (
            value, unit).
                - Value (float): The value of the patient's heart rate.
                - Unit (str): The unit of heart rate should be
                    "beats per minute".
            - "immobilization_for_3days" (boolean): Immobilization at least
            3 days OR surgery in the previous 4 weeks:
                - No = 0 points, Yes = +1.5 points.
            - "hemoptysis" (boolean): No = 0 points, Yes = +1 point.
            - "surgery_in_past4weeks" (boolean): Whether the patient has
            had a surgery for the past 4 weeks.
            - "malignancy_with_treatment" (boolean): Malignancy with
            treatment within 6 months or palliative:
                No = 0 points, Yes = +1 point.
            - "pe_number_one" (boolean): Hemoptysis: PE is #1 diagnosis OR
            equally likely: No = 0 points, Yes = +3 points.

    Returns:
        dict: Contains two key-value pairs:
            - "Explanation" (str): A detailed description of
            the calculation process.
            - "Answer" (float): The patient's score of Wells' criteria for
            Pulmonary Embolism.

    Notes:
        - None.

    Example:
        calculate_pe_wells_explanation({
            "previous_pe": False,
            "heart_rate": (78.0, 'beats per minute'),
            "immobilization_for_3days": False,
            "hemoptysis": False,
            "surgery_in_past4weeks": False,
            "malignancy_with_treatment": False,
            "pe_number_one": False,
            "previous_dvt": False
        })
        output: "{'Explanation': "\n    The criteria for the Wells' Criteria
        for Pulmonary Embolism score are listed below:
        \n\n        1. Clinical signs and symptoms of DVT:
        No = 0 points, Yes = +3 points\n
        2. PE is #1 diagnosis OR equally likely: No = 0 points, Yes = +3
        points\n
        3. Heart rate > 100: No = 0 points, Yes = +1.5 points\n
        4. Immobilization at least 3 days OR surgery in the previous 4
        weeks: No = 0 points, Yes = +1.5 points\n
        5. Previous, objectively diagnosed PE or DVT: No = 0 points,
        Yes = +1.5 points\n
        6. Hemoptysis: No = 0 points, Yes = +1 point\n
        7. Malignancy with treatment within 6 months or palliative: No = 0
        points, Yes = +1 point\n\n
        The total score is calculated by summing the points for each
        criterion.\\n\\n\n
        The Well's score for pulmonary embolism is currently 0.\nClinical
        signs and symptoms of DVT are not reported
        and so we assume that this is missing from the patient, keeping the
        current total at 0.
        Pulmonary Embolism is not reported to be the #1 diagnosis
        and so the total score remains unchanged,
        keeping the total score at 0.\nThe patient's heart rate is 78.0
        beats per minute.
        The heart rate is less than 100 bpm, and so the score
        remains unchanged,
        keeping the total score at 0.\nBecause the patient has not had an
        immobilization for at least 3 days,
        and the patient did not have a surgery in the past 4 weeks,
        the score remains at 0.
        \nBecause the patient has no previous diagnosis of pulmonary
        embolism (PE) or deep vein thrombosis (DVT),
        the score remains at 0.\nHemoptysis is reported to be absent
        and so the total score remains unchanged, keeping the total score at 0.
        \nMalignany with treatment within 6 months or
        palliative is reported to be absent
        and so the total score remains unchanged, keeping the total score at 0.
        \nThe patient's Well's score for pulmonary embolism is 0.\n",
        'Answer': 0}"
    """

    explanation = r"""
    The criteria for the Wells' Criteria for Pulmonary Embolism score
        are listed below:

        1. Clinical signs and symptoms of DVT: No = 0 points, Yes = +3 points
        2. PE is #1 diagnosis OR equally likely: No = 0 points, Yes = +3 points
        3. Heart rate > 100: No = 0 points, Yes = +1.5 points
        4. Immobilization at least 3 days OR surgery in the previous 4 weeks:
            No = 0 points, Yes = +1.5 points
        5. Previous, objectively diagnosed PE or DVT: No = 0 points,
            Yes = +1.5 points
        6. Hemoptysis: No = 0 points, Yes = +1 point
        7. Malignancy with treatment within 6 months or palliative:
            No = 0 points, Yes = +1 point

    The total score is calculated by summing the points for each criterion.\n\n
    """

    explanation += "The Well's score for pulmonary embolism is currently 0.\n"

    score = 0

    if 'clinical_dvt' in variables:
        if variables['clinical_dvt']:
            explanation += (
                f'Clinical signs and symptoms of DVT are reported to be '
                f'present and so three points are added to the score, '
                f'making the current total {score} + 3 = {score + 3}. '
            )
            score += 3
        else:
            explanation += (
                f'Clinical signs and symptoms of DVT are reported to be '
                f'absent and so the total score remains unchanged, keeping '
                f'the total score at {score}. '
            )
    else:
        explanation += (
            f'Clinical signs and symptoms of DVT are not reported and so we '
            f'assume that this is missing from the patient, keeping the '
            f'current total at {score}. '
        )

    if 'pe_number_one' in variables:
        if variables['pe_number_one']:
            explanation += (
                f'Pulmonary Embolism is reported to be the #1 diagnosis or '
                f'equally likely to be the #1 diagonsis and so we add 3 '
                f'points to the score making the current '
                f'total = {score} + 3 = {score + 3}.\n'
            )
            score += 3
        else:
            explanation += (
                f'Pulmonary Embolism is not reported to be the #1 diagnosis '
                f'and so the total score remains unchanged, keeping the '
                f'total score at {score}.\n'
            )
    else:
        explanation += (
            f'Whether Pulmonary Embolism is the #1 diagnosis or is equally '
            f'likely to being the #1 diagnosis is not reported and so we '
            f'assume this statement is false, '
            f'keeping the total unchanged at {score}.\n'
        )

    explanation += (
        f"The patient's heart rate is "
        f"{variables['heart_rate'][0]} beats per minute. "
    )
    if variables['heart_rate'][0] > 100:
        explanation += (
            f'The heart rate is more than 100 bpm, and so the score is '
            f'increased by 1.5, making the total score, {score} + 1.5 = '
            f'{score + 1.5}.\n'
        )
        score += 1.5
    else:
        explanation += (
            f'The heart rate is less than 100 bpm, and so the score remains '
            f'unchanged, keeping the total score at {score}.\n'
        )

    if 'immobilization_for_3days' not in variables:
        explanation += (
            "The report does not give an indication on whether the patient "
            "has had an immobilization for at least 3 days and so we assume "
            "this to be false."
        )
        variables['immobilization_for_3days'] = False

    if 'surgery_in_past4weeks' not in variables:
        explanation += (
            "The report does not give an indication on whether the patient "
            "has had a surgery for the past 4 weeks and "
            "so we assume this to be false."
        )
        variables['surgery_in_past4weeks'] = False

    if (
        not variables['immobilization_for_3days']
        and not variables['surgery_in_past4weeks']
    ):
        explanation += (
            f"Because the patient has not had an immobilization "
            f"for at least 3 days, and the patient did not have a surgery "
            f"in the past 4 weeks, the score remains at {score}.\n"
        )
    elif (
        not variables['immobilization_for_3days']
        and variables['surgery_in_past4weeks']
    ):
        explanation += (
            f'Because the patient did not have an immobilization for at '
            f'least 3 days but the patient had a surgery in the past 4 weeks, '
            f'the score increases to {score} + 1.5 = {score + 1.5}.\n'
        )
        score += 1.5
    elif (
        variables['immobilization_for_3days']
        and not variables['surgery_in_past4weeks']
    ):
        explanation += (
            f'Because the patient has had an immobilization for at least 3 '
            f'days but the patient did not have a surgery in the past 4 '
            f'weeks, the score increases to {score} + 1.5 = {score + 1.5}.\n'
        )
        score += 1.5
    elif (
        variables['immobilization_for_3days']
        and variables['surgery_in_past4weeks']
    ):
        explanation += (
            f'Because the patient has had an immobilization for at least 3 '
            f'days and the patient had a surgery in the past 4 weeks, '
            f'the score increases to {score} + 1.5 =  {score + 1.5}.\n'
        )
        score += 1.5

    if 'previous_pe' not in variables:
        explanation += (
            "The report does not give an indication on "
            "if the patient has previously had pulmonary embolism diagnosed "
            "and so we assume this to be false."
        )
        variables['previous_pe'] = False

    if 'previous_dvt' not in variables:
        explanation += (
            "The report does not give an indication on "
            "if the patient has previously been diagnosed with deep vein "
            "thrombosis and so we assume this to be false."
        )
        variables['previous_dvt'] = False

    if not variables['previous_pe'] and not variables['previous_dvt']:
        explanation += (
            f'Because the patient has no previous diagnosis of pulmonary '
            f'embolism (PE) or deep vein thrombosis (DVT), '
            f'the score remains at {score}.\n'
        )
    elif not variables['previous_pe'] and variables['previous_dvt']:
        explanation += (
            f'The patient not been diagnosed with pulmonary embolis (PE), '
            f'but the patient has previously been diagnosed with deep vein '
            f'thrombosis (DVT), we increase the current total by '
            f'1.5 so that {score} + 1.5 = {score + 1.5}.\n'
        )
        score += 1.5
    elif variables['previous_pe'] and not variables['previous_dvt']:
        explanation += (
            f'Because the patient has been previously diagnosed for '
            f'pulmonary embolism (PE), but the patient has never been '
            f'diagnosed for deep vein thrombosis (DVT), we increase the '
            f'current total by 1.5 so that {score} + 1.5 = {score + 1.5}.\n'
        )
        score += 1.5
    elif variables['previous_pe'] and variables['previous_dvt']:
        explanation += (
            f'Because the patient has previously been diagnosed for '
            f'pulmonary embolism (PE) and deep vein thrombosis (DVT), '
            f'we increase the current total by 1.5 '
            f'so that {score} + 1.5 = {score + 1.5}.\n'
        )
        score += 1.5

    if 'hemoptysis' in variables:
        if variables['hemoptysis']:
            explanation += (
                f'Hemoptysis is reported to be present and so one point is '
                f'incremented to the score, '
                f'making the current total {score} + 1 = {score + 1}.\n'
            )
            score += 1
        else:
            explanation += (
                f'Hemoptysis is reported to be absent '
                f'and so the total score remains unchanged, '
                f'keeping the total score at {score}.\n'
            )
    else:
        explanation += (
            f'Hemoptysis is not reported in the patient note '
            f'and so we assume that it is missing from the patient, '
            f'keeping the total score at {score}.\n'
        )

    if 'malignancy_with_treatment' in variables:
        if variables['malignancy_with_treatment']:
            explanation += (
                f'Malignany with treatment within 6 months or '
                f'palliative is reported to be present '
                f'and so one point is added to the score, '
                f'making the total score {score} + 1 =  {score + 1}.\n'
            )
            score += 1
        else:
            explanation += (
                f'Malignany with treatment within 6 months or '
                f'palliative is reported to be absent '
                f'and so the total score remains unchanged, '
                f'keeping the total score at {score}.\n'
            )
    else:
        explanation += (
            f'Malignany with treatment within 6 months or '
            f'palliative is not reported in the patient note '
            f'and so we assume that this is absent for the patient, '
            f'keeping the score at {score}.\n'
        )

    explanation += (
        f"The patient's Well's score for pulmonary embolism is {score}.\n"
    )

    return {"Explanation": explanation, "Answer": score}


if __name__ == "__main__":
    # Defining test cases
    test_cases = [
        {
            "previous_pe": False,
            "heart_rate": (78.0, 'beats per minute'),
            "immobilization_for_3days": False,
            "hemoptysis": False,
            "surgery_in_past4weeks": False,
            "malignancy_with_treatment": False,
            "pe_number_one": False,
            "previous_dvt": False,
        },
        {
            "previous_pe": False,
            "heart_rate": (106.0, 'beats per minute'),
            "immobilization_for_3days": False,
            "hemoptysis": False,
            "surgery_in_past4weeks": True,
            "malignancy_with_treatment": False,
            "pe_number_one": False,
            "previous_dvt": False,
        },
        {
            "previous_pe": False,
            "heart_rate": (129.0, 'beats per minute'),
            "immobilization_for_3days": False,
            "surgery_in_past4weeks": False,
            "clinical_dvt": True,
            "malignancy_with_treatment": False,
            "pe_number_one": True,
            "previous_dvt": False,
        },
    ]

    # Iterate the test cases and print the results
    for i, input_variables in enumerate(test_cases, 1):
        print(f"Test Case {i}: Input = {input_variables}")
        result = calculate_pe_wells_explanation(input_variables)
        print(result)
        print("-" * 50)



# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
r"""
This code is borrowed and modified based on the source code from the
'MedCalc-Bench' repository.
Original repository: https://github.com/ncbi-nlp/MedCalc-Bench

Modifications include:
- rewrite function compute_centor_score_explanation
- translation

Date: March 2025
"""

from .utils.age_conversion import (
    age_conversion_explanation,
)
from .utils.convert_temperature import (
    fahrenheit_to_celsius_explanation,
)


def compute_centor_score_explanation(input_variables):
    r"""
    Calculates the patient's Centor Score and generates a detailed
    explanatory text.

    Parameters:
        input_variables (dict): A dictionary containing the following
        key-value pairs:
            - "age" (tuple): The patient's albumin concentration in the
            format (value, unit).
                - Value (float): Age.
                - Unit (str): The unit can be "months", "years".
            - "temperature" (tuple): The patient's temperature in the
            format (value, unit).
                - Value (float): Temperature.
                - Unit (str): The unit can be "fahrenheit", "celsius".
            - "exudate_swelling_tonsils" (boolean): Exudate or swelling on
            tonsils
            - "tender_lymph_nodes" (boolean): Tender/swollen anterior
            cervical lymph nodes
            - "cough_absent" (boolean): Whether cough present

    Returns:
        dict: Contains two key-value pairs:
            - "Explanation" (str): A detailed description of
            the calculation process.
            - "Answer" (float): The patient's Centor Score.

    Notes:
        - None

    Example:
        compute_centor_score_explanation({
            "age": (48, "years"),
            "temperature": (99.0, "degrees fahreinheit"),
            "exudate_swelling_tonsils": False,
            "tender_lymph_nodes": False,
            "cough_absent": False,
        })

        output: "{'Explanation': "\n    The criteria listed in the Centor
        Score formula are listed below:\n    \n       1. Age: 3-14 years =
        +1 point, 15-44 years = 0 points, \n       ≥45 years = -1 point\n
        2. Exudate or swelling on tonsils: No = 0 points, Yes = +1
        point\n       3. Tender/swollen anterior cervical lymph nodes:
        \n       No = 0 points, Yes = +1 point\n       4. Temperature >38°C
        (100.4°F): No = 0 points, Yes = +1 point\n       5. Cough:
        Cough present = 0 points, Cough absent = +1 point\n    \n
        The Centor score is calculated by summing\n    the points for
        each criterion.\n\n\n    The current Centor score is 0.\nThe
        patient is 48 years old. Because the age is greater than 44 years,
        we decrease the score by one point, making the score 0 - 1 = -1.
        \nThe patient's temperature is 99.0 degrees fahrenheit. To convert
        to degrees celsius, apply the formula 5/9 * [temperature (degrees
        fahrenheit) - 32]. This means that the patient's temperature is
        5/9 * 67.0 = 37.222 degrees celsius. The patient's temperature is
        less than or equal to 38 degrees Celsius, and so we do not make any
        changes to the score, keeping the score at -1.\nThe patient note
        reports 'cough absent' as absent for the patient. Hence, we do not
        change the score, keeping the current score at -1.\nThe patient
        note reports 'tender/swollen anterior cervical lymph nodes' as
        absent for the patient. Hence, we do not change the score, keeping
        the current score at -1.\nThe patient note reports 'exudate or
        swelling on tonsils' as absent for the patient. Hence, we do not
        change the score, keeping the current score at -1.\nHence, the
        Centor score forthe patient is -1.\n", 'Answer': -1}"
    """

    explanation = """
    The criteria listed in the Centor Score formula are listed below:
    
       1. Age: 3-14 years = +1 point, 15-44 years = 0 points, 
       ≥45 years = -1 point
       2. Exudate or swelling on tonsils: No = 0 points, Yes = +1 point
       3. Tender/swollen anterior cervical lymph nodes: 
       No = 0 points, Yes = +1 point
       4. Temperature >38°C (100.4°F): No = 0 points, Yes = +1 point
       5. Cough: Cough present = 0 points, Cough absent = +1 point
    
    The Centor score is calculated by summing
    the points for each criterion.\n\n
    """

    centor_score = 0
    age_explanation, age = age_conversion_explanation(input_variables["age"])
    explanation += "The current Centor score is 0.\n"
    explanation += age_explanation

    if 3 <= age <= 14:
        explanation += (
            f"Because the age is between 3 and 14 years, we add "
            f"one point to the score making current score "
            f"{centor_score} + 1 = {centor_score + 1}.\n"
        )
        centor_score += 1
    elif 15 <= age <= 44:
        explanation += (
            f"Because the age is in between 15 and 44 years, "
            f"the score does not change, keeping the score at "
            f"{centor_score}.\n"
        )
    elif age >= 45:
        explanation += (
            f"Because the age is greater than 44 years, "
            f"we decrease the score by one point, making the "
            f"score {centor_score} - 1 = {centor_score - 1}.\n"
        )
        centor_score -= 1

    explanation_temp, temp_val = fahrenheit_to_celsius_explanation(
        input_variables["temperature"][0], input_variables["temperature"][1]
    )

    explanation += explanation_temp
    if temp_val > 38:
        explanation += (
            f"The patient's temperature is greater than 38 "
            f"degrees Celsius, and so we add one point to the "
            f"score, making the current score {centor_score} + 1 "
            f"= {centor_score + 1}.\n"
        )
        centor_score += 1
    elif temp_val <= 38:
        explanation += (
            f"The patient's temperature is less than or equal to "
            f"38 degrees Celsius, and so we do not make any "
            f"changes to the score, keeping the score at "
            f"{centor_score}.\n"
        )

    default_parameters_dict = {
        "cough_absent": "cough absent",
        "tender_lymph_nodes": "tender/swollen "
        "anterior cervical "
        "lymph nodes",
        "exudate_swelling_tonsils": "exudate or " "swelling on " "tonsils",
    }

    for parameter in default_parameters_dict:
        if parameter not in input_variables:
            explanation += (
                f"The patient note does not mention details "
                f"about '{default_parameters_dict[parameter]}' "
                f"and so we assume it to be absent. "
            )
            input_variables[parameter] = False
            explanation += (
                f"Hence, we do not change the score, keeping the "
                f"current score at {centor_score}.\n"
            )
        elif not input_variables[parameter]:
            explanation += (
                f"The patient note reports '"
                f"{default_parameters_dict[parameter]}' as "
                f"absent for the patient. Hence, "
                f"we do not change the score, "
                f"keeping the current score at {centor_score}.\n"
            )
        else:
            explanation += (
                f"The patient note reports '"
                f"{default_parameters_dict[parameter]}' as "
                f"present for the patient. "
                f"Hence, we increase the score by 1, "
                f"making the current score "
                f"{centor_score} + 1 = {centor_score + 1}.\n"
            )
            centor_score += 1

    explanation += (
        f"Hence, the Centor score for" f"the patient is {centor_score}.\n"
    )

    return {"Explanation": explanation, "Answer": centor_score}


if __name__ == "__main__":
    # Defining test cases
    test_cases = [
        {
            "age": (48, "years"),
            "temperature": (99.0, "degrees fahreinheit"),
            "exudate_swelling_tonsils": False,
            "tender_lymph_nodes": False,
            "cough_absent": False,
        }
    ]

    # Iterate the test cases and print the results
    for i, input_variables in enumerate(test_cases, 1):
        print(f"Test Case {i}: Input = {input_variables}")
        result = compute_centor_score_explanation(input_variables)
        print(result)
        print("-" * 50)



# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
r"""
This code is borrowed and modified based on the source code from the
'MedCalc-Bench' repository.
Original repository: https://github.com/ncbi-nlp/MedCalc-Bench

Modifications include:
- rewrite function targetweight_explanation
- translation

Date: March 2025
"""

from datetime import datetime


def compute_gestational_age_explanation(input_parameters):
    r"""
    Calculates the patient's gestational age and generates a detailed
    explanatory text.

    Parameters:
        input_parameters (dict): A dictionary containing the following
        key-value pairs:
            - "current_date" (date): The current date in the format "%m/%d/%Y".
            - "menstrual_date" (date): The patient's menstrual date in the
            format "%m/%d/%Y".

    Returns:
        dict: Contains two key-value pairs:
            - "Explanation" (str): A detailed description of
            the calculation process.
            - "Answer" (float): The patient's gestational age.

    Notes:
        - None

    Example:
        compute_gestational_age_explanation({"current_date": "04/29/2022",
        "menstrual_date": "01/06/2022"})

        output: "{'Explanation': "To compute the estimated gestational age,
        we compute the number of weeks and days apart today's date is from
        the patient's last menstrual period date. The current date is
        04/29/2022 and the patient's last menstrual period date was
        01/06/2022. The gap between these two dates is 16 weeks and 1 days.
        Hence, the estimated gestational age is 16 weeks and 1 days. ",
        'Answer': ('16 weeks', '1 days')}"
    """

    date2 = input_parameters["current_date"]
    date1 = input_parameters["menstrual_date"]

    explanation = (
        "To compute the estimated gestational age, we compute the number of "
        "weeks and days apart today's date is from the patient's last "
        "menstrual period date. "
    )
    explanation += (
        f"The current date is {date2} and the patient's last "
        f"menstrual period date was {date1}. "
    )

    datetime1 = datetime.strptime(date1, "%m/%d/%Y")
    datetime2 = datetime.strptime(date2, "%m/%d/%Y")

    delta = abs(datetime2 - datetime1)

    weeks = delta.days // 7
    days = delta.days % 7

    if weeks == 0:
        explanation += (
            f"The gap between these two dates is {days} days. "
            f"Hence, the estimated gestational age is {days} days. "
        )
    elif days == 0:
        explanation += (
            f"The gap between these two dates is {weeks} weeks. "
            f"Hence, the estimated gestational age is {weeks} weeks. "
        )
    else:
        explanation += (
            f"The gap between these two dates is {weeks} weeks and {days} "
            f"days. Hence, the estimated gestational age is {weeks} weeks and "
            f"{days} days. "
        )

    return {
        "Explanation": explanation,
        "Answer": (f"{weeks} weeks", f"{days} days"),
    }


if __name__ == "__main__":
    # Defining test cases
    test_cases = [
        {
            "current_date": "04/29/2022",
            "menstrual_date": "01/06/2022",
        },
        {
            "current_date": "11/15/2005",
            "menstrual_date": "06/16/2005",
        },
    ]

    # {'Current Date': '04/29/2022', 'Last menstrual date': '01/06/2022'}
    # {'Current Date': '11/15/2005', 'Last menstrual date': '06/16/2005'}

    # Iterate the test cases and print the results
    for i, input_variables in enumerate(test_cases, 1):
        print(f"Test Case {i}: Input = {input_variables}")
        result = compute_gestational_age_explanation(input_variables)
        print(result)
        print("-" * 50)



# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
r"""
This code is borrowed and modified based on the source code from the
'MedCalc-Bench' repository.
Original repository: https://github.com/ncbi-nlp/MedCalc-Bench

Modifications include:
- rewrite function add_40_weeks_explanation
- translation

Date: March 2025
"""

from datetime import datetime, timedelta


def add_40_weeks_explanation(input_data):
    r"""
    Calculates the patient's estimated due date and generates a detailed
    explanatory text.

    Parameters:
        input_data (dict): A dictionary containing the following key-value
        pairs:
            - "cycle_length" (int): The cycle length of the patient's
            menstrual period.
            - "menstrual_date" (date): The patient's menstrual date in the
            format "%m/%d/%Y".

    Returns:
        dict: Contains two key-value pairs:
            - "Explanation" (str): A detailed description of
            the calculation process.
            - "Answer" (float): The patient's estimated due date.

    Notes:
        - None

    Example:
        add_2_weeks_explanation({"cycle_length": 21,
        "menstrual_date": "09/17/2011",})

        output: "{'Explanation': "The patient's estimated due date based on
        their last period is computed by using Naegele's Rule. Using
        Naegele's Rule, we add 40 weeks to the patient's last menstrual
        period date. We then add or subtract days from the patient's
        estimated due date depending on how many more or less days a
        patient's cycle length is from the standard 28 days. \nThe patient's
        last menstrual period was 09/17/2011. \nThe date after adding 40
        weeks to the patient's last menstrual period date is 06/23/2012.
        \nBecause the patient's cycle length is 21 days, this means that we
        must subtract 7 days from the patient's estimate due date. Hence,
        the patient's estimated due date is 06/30/2012. \n", 'Answer':
        '06/30/2012'}"
    """

    input_date_str = input_data["menstrual_date"]
    cycle_length = input_data["cycle_length"]

    explanation = (
        "The patient's estimated due date based on their last "
        "period is computed by using Naegele's Rule. "
    )
    explanation += (
        "Using Naegele's Rule, we add 40 weeks to the patient's last "
        "menstrual period date. We then add or subtract days from the "
        "patient's estimated due date depending on how many more or less "
        "days a patient's cycle length is from the standard 28 days. \n"
    )
    explanation += (
        f"The patient's last menstrual period was {input_date_str}. \n"
    )

    input_date = datetime.strptime(input_date_str, "%m/%d/%Y")
    future_date = input_date + timedelta(weeks=40)

    explanation += (
        f"The date after adding 40 weeks to the patient's last "
        f"menstrual period date is "
        f"{future_date.strftime('%m/%d/%Y')}. \n"
    )

    if cycle_length == 28:
        explanation += (
            f"Because the patient's cycle length is 28 days, we do not make "
            f"any changes to the date. Hence, the patient's estimated due "
            f"date is {future_date.strftime('%m/%d/%Y')}. \n"
        )
    elif cycle_length < 28:
        cycle_length_gap = abs(cycle_length - 28)
        future_date = future_date + timedelta(days=cycle_length_gap)
        explanation += (
            f"Because the patient's cycle length is {abs(cycle_length)} days, "
            f"this means that we must subtract {cycle_length_gap} days from "
            f"the patient's estimate due date. Hence, the patient's "
            f"estimated due date is {future_date.strftime('%m/%d/%Y')}. \n"
        )
    elif cycle_length > 28:
        cycle_length_gap = abs(cycle_length - 28)
        future_date = future_date + timedelta(days=cycle_length_gap)
        explanation += (
            f"Because the patient's cycle length is {cycle_length} days, "
            f"this means that we must add {cycle_length_gap} days to the "
            f"patient's estimate due date. Hence, the patient's estimated "
            f"due date is {future_date.strftime('%m/%d/%Y')}. \n"
        )

    return {
        "Explanation": explanation,
        "Answer": future_date.strftime('%m/%d/%Y'),
    }


if __name__ == "__main__":
    # Defining test cases
    test_cases = [
        {
            "cycle_length": 21,
            "menstrual_date": "09/17/2011",
        },
        {
            "cycle_length": 28,
            "menstrual_date": "12/03/2023",
        },
    ]

    # {'cycle length': 21, 'Last menstrual date': '09/17/2011'}
    # {'cycle length': 28, 'Last menstrual date': '12/03/2023'}

    # Iterate the test cases and print the results
    for i, input_variables in enumerate(test_cases, 1):
        print(f"Test Case {i}: Input = {input_variables}")
        result = add_40_weeks_explanation(input_variables)
        print(result)
        print("-" * 50)



# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
r"""
This code is borrowed and modified based on the source code from the
'MedCalc-Bench' repository.
Original repository: https://github.com/ncbi-nlp/MedCalc-Bench

Modifications include:
- None

Date: March 2025
"""

from .utils.rounding import round_number


def mean_arterial_pressure_explanation(input_variables):
    r"""
    Calculates the patient's mean arterial pressure and
    generates a detailed explanatory text.

    Parameters:
        input_variables (dict): A dictionary containing the following
        key-value pairs:
            - "sys_bp" (array): The patient's systolic blood pressure (
            value, unit).
                - Value (float): Systolic blood pressure.
                - Unit (str): The unit of systolic blood pressure,
                which can be 'mm hg'.
            - "dia_bp" (array): The patient's diastolic blood pressure (
            value, unit).
                - Value (float): Diastolic blood pressure.
                - Unit (str): The unit of diastolic blood pressure,
                which can be 'mm hg'.

    Returns:
        dict: Contains two key-value pairs:
            - "Explanation" (str): A detailed description of
            the calculation process.
            - "Answer" (float): The patient's mean arterial pressure.

    Notes:
        - None

    Example:
        mean_arterial_pressure_explanation({"sys_bp": [120.0, 'mm hg'],
        "dia_bp": [80.0, 'mm hg']})

        output: "{'Explanation': "The mean average pressure is computed by
        the formula 2/3 * (diastolic blood pressure) + 1/3 *
        (systolic blood pressure). Plugging in the values,
        we get 2/3 * 80.0 mm Hg + 1/3 * 120.0 mm Hg = 93.333 mm Hg.
        \nHence, the patient's mean arterial pressure is 93.333 mm Hg.\n",
        'Answer': 93.333}"
    """

    sys_bp = input_variables['sys_bp']
    dia_bp = input_variables['dia_bp']

    output = ""

    value = round_number(2 * dia_bp[0] / 3 + sys_bp[0] / 3)

    output += (
        f"The mean average pressure is computed by the formula 2/3 * ("
        f"diastolic blood pressure) + 1/3 * (systolic blood pressure). "
        f"Plugging in the values, we get 2/3 * {dia_bp[0]} mm Hg + "
        f"1/3 * {sys_bp[0]} mm Hg = {value} mm Hg.\n"
    )
    output += (
        f"Hence, the patient's mean arterial pressure is {value} mm Hg.\n"
    )

    return {"Explanation": output, "Answer": value}


if __name__ == "__main__":
    # Defining test cases
    test_cases = [
        {
            "sys_bp": [120.0, 'mm hg'],  # Systolic Blood Pressure
            "dia_bp": [80.0, 'mm hg'],  # Diastolic Blood Pressure
        }
    ]

    # Iterate the test cases and print the results
    for i, input_variables in enumerate(test_cases, 1):
        print(f"Test Case {i}: Input = {input_variables}")
        result = mean_arterial_pressure_explanation(input_variables)
        print(result)
        print("-" * 50)



# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
r"""
This code is borrowed and modified based on the source code from the
'MedCalc-Bench' repository.
Original repository: https://github.com/ncbi-nlp/MedCalc-Bench

Modifications include:
- rewrite function add_2_weeks_explanation
- translation

Date: March 2025
"""

from datetime import datetime, timedelta


def add_2_weeks_explanation(input_data):
    r"""
    Calculates the patient's estimated conception date and generates a
    detailed explanatory text.

    Parameters:
        input_data (dict): A dictionary containing the following key-value
        pairs:
            - "cycle_length" (int): The cycle length of the patient's
            menstrual period.
            - "menstrual_date" (date): The patient's menstrual date in the
            format "%m/%d/%Y".

    Returns:
        dict: Contains two key-value pairs:
            - "Explanation" (str): A detailed description of
            the calculation process.
            - "Answer" (float): The patient's estimated conception date.

    Notes:
        - None

    Example:
        add_2_weeks_explanation({"cycle_length": 20, "menstrual_date":
        "01/21/2004"})

        output: "{'Explanation': "The patient's estimated date of
        conception based on their last period is computed by adding to 2
        weeks to the patient's last menstrual period date. The patient's
        last menstrual period was 01/21/2004.
        \nHence, the estimated date of conception after adding 2 weeks to
        the patient's last menstrual period date is 02/04/2004. \n",
        'Answer': '02/04/2004'}"
    """

    input_date_str = input_data["menstrual_date"]
    # cycle_length = input_data["cycle_length"]

    explanation = (
        "The patient's estimated date of conception based on their last "
        "period is computed by adding to 2 weeks to the patient's last "
        "menstrual period date. "
    )
    explanation += (
        f"The patient's last menstrual period was {input_date_str}. \n"
    )

    input_date = datetime.strptime(input_date_str, "%m/%d/%Y")
    future_date = input_date + timedelta(weeks=2)

    explanation += (
        f"Hence, the estimated date of conception after adding 2 weeks "
        f"to the patient's last menstrual period date is "
        f"{future_date.strftime('%m/%d/%Y')}. \n"
    )

    return {
        "Explanation": explanation,
        "Answer": future_date.strftime('%m/%d/%Y'),
    }


if __name__ == "__main__":
    # Defining test cases
    test_cases = [
        {
            "cycle_length": 20,
            "menstrual_date": "01/21/2004",
        },
        {
            "cycle_length": 25,
            "menstrual_date": "11/06/2005",
        },
    ]

    # {'cycle length': 20, 'Last menstrual date': '01/21/2004'}
    # {'cycle length': 25, 'Last menstrual date': '11/06/2005'}

    # Iterate the test cases and print the results
    for i, input_variables in enumerate(test_cases, 1):
        print(f"Test Case {i}: Input = {input_variables}")
        result = add_2_weeks_explanation(input_variables)
        print(result)
        print("-" * 50)



# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
r"""
This code is borrowed and modified based on the source code from the
'MedCalc-Bench' repository.
Original repository: https://github.com/ncbi-nlp/MedCalc-Bench

Modifications include:
- rewrite function generate_cockcroft_gault_explanation
- translation

Date: March 2025
"""

from . import (
    adjusted_body_weight,
    bmi_calculator,
    ideal_body_weight,
)
from .utils.age_conversion import (
    age_conversion_explanation,
)
from .utils.rounding import round_number
from .utils.unit_converter_new import (
    conversion_explanation,
)
from .utils.weight_conversion import (
    weight_conversion_explanation,
)


def generate_cockcroft_gault_explanation(params):
    r"""
    Calculates the patient's Creatinine Clearance and generates a detailed
    explanatory text.

    Parameters:
        params (dict): A dictionary containing the following key-value pairs:
            - "sex" (str): The patient's gender, which can be either "Male"
            or "Female".
            - "weight" (tuple): The patient's weight information in the
            format (value, unit).
                - Value (float): The numerical weight measurement.
                - Unit (str): The unit of weight, which can be "lbs" (
                pounds), "g" (grams), or "kg" (kilograms).
            - "height" (tuple): The patient's height information in the
            format (value, unit).
                - Value (float): The numerical height measurement.
                - Unit (str): The unit of height,
                which can be "cm" (centimeters) or "in" (inches).
            - "creatinine" (tuple): The patient's creatinine information in the
            format (value, unit).
                - Value (float): The value of creatinine.
                - Unit (str): The unit of creatinine,
                which can be "mg/dL", "μmol/L", and so on.
            - "age" (array): The patient's albumin concentration in the
            format (value, unit).
                - Value (float): Age.
                - Unit (str): The unit can be "months", "years".

    Returns:
        dict: Contains two key-value pairs:
            - "Explanation" (str): A detailed description of
            the calculation process.
            - "Answer" (float): The patient's Creatinine Clearance.

    Notes:
        - Use the `weight_conversion_explanation` function to
        convert weight to kilogram.
        - Use the `bmi_calculator` function to calculate BMI.
        - Use the `ideal_body_weight` function to calculate Ideal Body
        Weight (IBW)

    Example:
        generate_cockcroft_gault_explanation({"sex": "Female",
        "weight": (55.0, "kg"),"height": (162.8, "cm"),
        "creatinine": (0.57, "mg/dL"),"age": (16, "years")})

        output: "{'Explanation': "The formula for computing Cockcroft-Gault
        is given by CrCl = ((140 - age) * adjusted weight * (
        gender_coefficient)) / (serum creatinine * 72), where the
        gender_coefficient is 1 if male, and 0.85 if female. The serum
        creatinine concentration is in mg/dL.\nThe patient's gender is
        female, which means that the gender coefficient is 0.85.\nThe
        patient is 16 years old. \nThe concentration of creatinine is 0.57
        mg/dL. \nThe formula for computing the patient's BMI is
        (weight)/(height * height), where weight is the patient's weight in
        kg and height is the patient's height in m. \nThe patient's height
        is 162.8 cm, which is 162.8 cm * 1 m / 100 cm = 1.628 m. The
        patient's weight is 55.0 kg. The patient's bmi is therefore 55.0 kg
        / (1.628 m * 1.628 m) = 20.752 kg/m^2. The patient's BMI is 20.8,
        indicating they are normal weight.\nBecause the patient is normal,
        we take minimum of the ideal body weight and the patient's body as
        the patient's adjusted weight for the Cockroft-Gault Equation.
        Hence, the adjusted body weight is the minimum of the two giving us
        an adjusted body weight of 54.918 kg.\n\nUsing the Cockcroft-Gault
        equation:\nCrCl = ((140 - age) * adjusted weight *
        gender_coefficient) / (serum creatinine * 72). \nPlugging the
        patient's values gives us ((140 - 16) * 54.918 * 0.85) /
        (0.57 * 72) = 141.042 mL/min. Hence, the patient's creatinine
        clearance is 141.042 mL/min.\n", 'Answer': 141.042}"
    """

    weight_exp, weight = weight_conversion_explanation(params["weight"])

    output = (
        "The formula for computing Cockcroft-Gault is given by CrCl = ((140 "
        "- age) * adjusted weight * (gender_coefficient)) / (serum "
        "creatinine * 72), where the gender_coefficient is 1 if male, "
        "and 0.85 if female. The serum creatinine concentration is in mg/dL.\n"
    )
    output += f"The patient's gender is {params['sex'].lower()}, "
    gender_coefficient = 1 if params["sex"] == "Male" else 0.85
    output += (
        f"which means that the gender coefficient is {gender_coefficient}.\n"
    )
    age_explanation, age = age_conversion_explanation(params["age"])

    output += f"{age_explanation}\n"

    serum_creatinine_value = params['creatinine'][0]
    serum_creatinine_units = params['creatinine'][1]
    is_male = True if params["sex"] == "Male" else False

    bmi_response = bmi_calculator.bmi_calculator_explanation(params)
    bmi = float(bmi_response["Answer"])

    if bmi < 18.5:
        weight_status = "underweight"
    elif 18.5 <= bmi <= 24.9:
        weight_status = "normal weight"
    else:
        weight_status = "overweight/obese"

    ideal_weight_response = ideal_body_weight.ibw_explanation(params)
    adjusted_weight_response = adjusted_body_weight.abw_explanation(params)
    serum_creatinine_explanation, serum_creatinine = conversion_explanation(
        serum_creatinine_value,
        "creatinine",
        113.12,
        None,
        serum_creatinine_units,
        "mg/dL",
    )

    output += serum_creatinine_explanation + "\n"

    output += f"{bmi_response['Explanation']}"
    output += (
        f"The patient's BMI is {bmi:.1f}, indicating they are "
        f"{weight_status}.\n"
    )

    adjusted_weight = 0

    if bmi < 18.5:
        output += (
            f"Because the patient is underweight, we take the patient's "
            f"weight, {weight} kg as the patient's adjusted weight needed "
            f"for the Cockroft-Gault Equation. "
        )
        adjusted_weight = weight
    elif 18.5 <= bmi <= 24.9:
        adjusted_weight = min(ideal_weight_response["Answer"], weight)
        output += (
            "Because the patient is normal, we take minimum of the ideal "
            "body weight and the patient's body as the patient's adjusted "
            "weight for the Cockroft-Gault Equation. "
        )
        output += (
            f"Hence, the adjusted body weight is the minimum of the two "
            f"giving us an adjusted body weight of {adjusted_weight} kg.\n"
        )

    else:
        output += (
            "Because the patient is overweight/obese, we use the adjusted "
            "body weight formula to get the adjusted weight used for "
            "Cockroft-Gault Equation. "
        )
        output += (
            "Shown below is the computation for IBW (ideal body weight).\n"
        )
        output += f"{ideal_weight_response['Explanation']}"
        output += (
            "Shown below is the computation for ABW (adjusted body weight).\n"
        )
        output += f"{adjusted_weight_response['ABW']}"
        adjusted_weight = adjusted_weight_response["Answer"]

    # Calculate creatinine clearance
    if is_male:
        constant = 1
    else:
        constant = 0.85

    creatinine_clearance = round_number(
        ((140 - age) * adjusted_weight * constant) / (serum_creatinine * 72)
    )

    # Explanation of Cockcroft-Gault equation and result
    output += "\nUsing the Cockcroft-Gault equation:\n"
    output += (
        "CrCl = ((140 - age) * adjusted weight * gender_coefficient) "
        "/ (serum creatinine * 72).\n"
    )
    output += (
        f"Plugging the patient's values gives us ((140 - {age}) * "
        f"{adjusted_weight} * {gender_coefficient}) / "
        f"({serum_creatinine} * 72) = {creatinine_clearance} mL/min. "
    )
    output += (
        f"Hence, the patient's creatinine clearance is "
        f"{creatinine_clearance} mL/min.\n"
    )

    return {"Explanation": output, "Answer": creatinine_clearance}


if __name__ == "__main__":
    # Defining test cases
    test_cases = [
        {
            "sex": "Female",
            "weight": (55.0, "kg"),
            "height": (162.8, "cm"),
            "creatinine": (0.57, "mg/dL"),
            "age": (16, "years"),
        },
        {
            "sex": "Male",
            "weight": (68.0, "kg"),
            "height": (176.0, "cm"),
            "creatinine": (1.0, "mg/dL"),
            "age": (56, "years"),
        },
    ]
    # {'sex': 'Female', 'weight': [55.0, 'kg'], 'height': [162.8, 'cm'],
    # 'creatinine': [0.57, 'mg/dL'], 'age': [16, 'years']}

    # Iterate the test cases and print the results
    for i, input_variables in enumerate(test_cases, 1):
        print(f"Test Case {i}: Input = {input_variables}")
        result = generate_cockcroft_gault_explanation(input_variables)
        print(result)
        print("-" * 50)



# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
r"""
This code is borrowed and modified based on the source code from the
'MedCalc-Bench' repository.
Original repository: https://github.com/ncbi-nlp/MedCalc-Bench

Modifications include:
- rewrite function caprini_score_explanation
- translation

Date: March 2025
"""

from .utils.age_conversion import (
    age_conversion_explanation,
)

param_full_name = {
    "surgery_type": "type of surgery",
    "major_surgery": ("major surgery in the last month", 1),
    "chf": ("congestive heart failure in the last month", 1),
    "sepsis": ("sepsis in the last month", 1),
    "pneumonia": ("pneumonia in the last month", 1),
    "immobilizing_plaster_case": (
        "immobilizing plaster cast in the last month",
        2,
    ),
    "hip_pelvis_leg_fracture": (
        "hip, pelvis, or leg fracture in the last month",
        5,
    ),
    "stroke": ("stroke in the last month", 5),
    "multiple trauma": ("multiple trauma in the last month", 5),
    "acute_spinal_chord_injury": (
        "acute spinal cord injury causing paralysis in the last month",
        5,
    ),
    "varicose_veins": ("varicose veins", 1),
    "current_swollen_legs": ("current swollen legs", 1),
    "current_central_venuous": ("current central venuous access", 2),
    "previous_dvt": ('previous DVT documented', 3),
    "previous_pe": ('previous pulmonary embolism documented', 3),
    "family_history_thrombosis": ("family history of thrombosis", 3),
    "positive_factor_v": ("Positive Factor V Leiden", 3),
    "positive_prothrombin": ("Positive prothrombin 20210A", 3),
    "serum_homocysteine": ("elevated serum homocysteine", 3),
    "positive_lupus_anticoagulant": ("positive lupus anticoagulant", 3),
    "elevated_anticardiolipin_antibody": (
        "elevated anticardiolipin antibody",
        3,
    ),
    "heparin_induced_thrombocytopenia": (
        "heparin-induced thrombocytopenia",
        3,
    ),
    "congenital_acquired_thrombophilia": (
        "Other congenital or acquired thrombophilia",
        3,
    ),
    "mobility": "mobility",
    "inflammatory_bowel_disease": ("history of inflammatory bowel disease", 1),
    "acute_myocardial_infarction": ("acute Myocardial infarction", 1),
    "copd": ("chronic Obstructive Pulmonary Disease", 1),
    "malignancy": ("malignancy", 2),
    "bmi": "bmi",
}

surgery_type = {
    "none": 0,
    "minor": 1,
    "major": 2,
    "laparoscopic": 2,
    "arthroscopic": 2,
    "elective major lower extremity arthroplasty": 5,
}
mobility = {"normal": 0, "on bed rest": 1, "confined to bed >72 hours": 2}


def caprini_score_explanation(input_parameters):
    explanation = """
    The criteria for the Caprini Score are listed below:
  
     1. Age, years: ≤40 = 0 points, 41-60 = +1 point, 61-74 = +2 points, 
     ≥75 = +3 points
     2. Sex: Male = 0 points, Female = +1 point
     3. Type of surgery: None = 0 points, Minor = +1 point, Major >45 min (
     laparoscopic or arthroscopic) = +2 points, Elective major 
     lower extremity arthroplasty = +5 points
     4. Recent (≤1 month) event: Major surgery = +1 point, 
     Congestive heart failure (CHF) = +1 point, Sepsis = +1 point, 
     Pneumonia = +1 point, Immobilizing plaster cast = +1 point, 
     Hip, pelvis, or leg fracture = +5 points, Stroke = +5 points, 
     Multiple trauma = +5 points, Acute spinal 
     cord injury causing paralysis = +5 points
     5. Venous disease or clotting disorder: Varicose 
     veins = +1 point, Current swollen legs = +1 point, 
     Current central venous access = +2 points, History of 
     deep vein thrombosis (DVT) or pulmonary embolism (PE) = +3 
     points, Family history of thrombosis = +3 points, 
     Positive Factor V Leiden = +3 points, Positive prothrombin 
     20210A = +3 points, Elevated serum homocysteine = +3 points
     6. Other congenital or acquired thrombophilia: Positive 
     lupus anticoagulant = +3 points, Elevated anticardiolipin 
     antibody = +3 points, Heparin-induced thrombocytopenia = +3 points
     7. Mobility: Normal, out of bed = 0 points, Medical patient currently 
     on bed rest = +1 point, Patient confined to bed >72 hours = +2 points
     8. Other present and past history: History of inflammatory bowel 
     disease = +1 point, BMI ≥25 = +1 point, Acute myocardial infarction = 
     +1 point, Chronic obstructive pulmonary disease (COPD) = +1 point, 
     Present or previous malignancy = +2 points
  
    The total Caprini Score is calculated by summing 
    the points for each criterion.\n\n
    """

    explanation += "The patient's current caprini score is 0.\n"
    score = 0

    gender = input_parameters["sex"]

    explanation += f"The patient's gender is {gender}.\n"

    age_exp, age = age_conversion_explanation(input_parameters["age"])
    explanation += age_exp

    if age <= 40:
        explanation += (
            f"Because the patient's age is less or equal to 40, "
            f"we do not add any points to the total, keeping the "
            f"current total at {score}.\n"
        )
    elif 41 <= age <= 60:
        explanation += (
            f"Because the patient's age is between 61 and 74, "
            f"we add one point to the current total, making the "
            f"current total, {score} + 1 = {score + 1}.\n"
        )
        score += 1
    elif 61 <= age <= 74:
        explanation += (
            f"Because the patient's age is between 61 and 74, "
            f"we add two points to the current total, making the "
            f"current total, {score} + 2 = {score + 2}.\n"
        )
        score += 2
    elif age >= 75:
        explanation += (
            f"Because the patient's age at least 75, "
            f"we add three points to the current total, "
            f"making the current total, "
            f"{score} + 3 = {score + 3}.\n"
        )
        score += 3

    for param, value in param_full_name.items():
        if param not in input_parameters:
            explanation += (
                f"The patient does not report anything about"
                f" {param_full_name[param][0]} and so we assume "
                f"this to be false. Hence, 0 points are added to "
                f"the score, keeping the total at {score}. "
            )

        elif param == "mobility":
            value = input_parameters[param]

            explanation += (
                f"The patient's mobility status is '{value}'. "
                f"Hence, we add {mobility[value]} points to the "
                f"total, making the current total "
                f"{mobility[value]} + {score} = "
                f"{mobility[value] + score}.\n "
            )
            score += mobility[value]

        elif param == "surgery_type":
            value = input_parameters[param]
            explanation += (
                f"The patient's surgery type is reported to be "
                f"'{value}'. Hence, we add {surgery_type[value]} "
                f"points to the total, making the current total"
                f" {surgery_type[value]} + {score} = "
                f"{surgery_type[value] + score}.\n "
            )
            score += surgery_type[value]

        elif param == "bmi":
            if input_parameters["bmi"][0] > 25:
                explanation += (
                    f"The patient's BMI is "
                    f"{input_parameters['bmi'][0]} kg/m^2 which "
                    f"is greater than 25 kg/m^2, and so we add "
                    f"2 points to the total, making the "
                    f"current total {score} + 2 = {score + 2}.\n"
                )
                score += 2
            else:
                explanation += (
                    f"The patient's BMI is "
                    f"{input_parameters['bmi'][0]} kg/m^2 "
                    f"which is less than 25 kg/m^2, and "
                    f"so we add 0 points to the total, "
                    f"keeping the total at {score}.\n"
                )

        elif input_parameters[param]:
            points = param_full_name[param][1]
            explanation += (
                f"The patient's has {param_full_name[param][0]}. "
                f"Hence, we add {points} to the total, "
                f"making the current total {points} + "
                f"{score} = {int(points) + score}.\n "
            )

        elif not input_parameters[param]:
            points = param_full_name[param][1]
            explanation += (
                f"The patient's has does not have "
                f"{param_full_name[param][0]}. Hence, "
                f"0 points are added to the score, "
                f"keeping the total at {score}.\n"
            )

    return {"Explanation": explanation, "Answer": score}


if __name__ == "__main__":
    # Defining test cases
    test_cases = [
        {
            'sex': 'Male',
            'age': (17, 'years'),
            # "mobility": "normal",  # Optional
            # "surgery_type": "major",  # Optional
            # "bmi": (22.0, "kg/m^2"),  # Optional
            # "major_surgery": True,  # Optional
            # "chf": False,  # Optional
            # "sepsis": False,  # Optional
            # "pneumonia": False,  # Optional
            # "immobilizing_plaster_case": False,  # Optional
            # "hip_pelvis_leg_fracture": False,  # Optional
            # "stroke": False,  # Optional
            # "multiple trauma": False,  # Optional
            # "acute_spinal_chord_injury": False,  # Optional
            # "varicose_veins": False,  # Optional
            # "current_swollen_legs": False,  # Optional
            # "current_central_venuous": False,  # Optional
            # "previous_dvt": False,  # Optional
            # "previous_pe": False,  # Optional
            # "family_history_thrombosis": False,  # Optional
            # "positive_factor_v": False,  # Optional
            # "positive_prothrombin": False,  # Optional
            # "serum_homocysteine": False,  # Optional
            # "positive_lupus_anticoagulant": False,  # Optional
            # "elevated_anticardiolipin_antibody": False,  # Optional
            # "heparin_induced_thrombocytopenia": False,  # Optional
            # "congenital_acquired_thrombophilia": False,  # Optional
            # "inflammatory_bowel_disease": False,  # Optional
            # "acute_myocardial_infarction": False,  # Optional
            # "copd": False,  # Optional
            # "malignancy": False,  # Optional
        },
    ]

    # Iterate the test cases and print the results
    for i, input_variables in enumerate(test_cases, 1):
        print(f"Test Case {i}: Input = {input_variables}")
        result = caprini_score_explanation(input_variables)
        print(result)
        print("-" * 50)



# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
r"""
This code is borrowed and modified based on the source code from the
'MedCalc-Bench' repository.
Original repository: https://github.com/ncbi-nlp/MedCalc-Bench

Modifications include:
- rewrite function maintenance_fluid_explanation
- translation

Date: March 2025
"""

from .utils.rounding import round_number
from .utils.weight_conversion import (
    weight_conversion_explanation,
)


def maintenance_fluid_explanation(input_parameters):
    r"""
    Calculates the patient's maintenance fluid in mL/hr and
    generates a detailed explanatory text.

    Parameters:
        input_parameters (dict): A dictionary containing the following
        key-value pairs:
            - "weight" (tuple): The patient's weight information in the
            format (value, unit).
                - Value (float): The numerical weight measurement.
                - Unit (str): The unit of weight, which can be "lbs"
                (pounds), "g" (grams), or "kg" (kilograms).

    Returns:
        dict: Contains two key-value pairs:
            - "Explanation" (str): A detailed description of
            the calculation process.
            - "Answer" (float): The patient's maintenance fluid in mL/hr.

    Notes:
        - Uses the `weight_conversion_explanation` function to convert
        weight to kilogram.

    Example:
        maintenance_fluid_explanation(
        {
            "weight": (12.0, "kg"),  # weight 150 lbs
        })

        output: "{'Explanation': "The patient's weight is 12.0 kg. For
        patient's whose weight is in between 10 kg and 20 kg, the formula
        for computing maintenance fluid is 40 mL/hr + 2 mL/kg/hr * (weight (
        in kilograms) - 10 kilograms). Hence, plugging into this formula,
        we get 40 mL/hr + 2 mL/kg/hr * (12.0 kg - 10 kg) = 44.0
        mL/hr.\nHence, the patient's fluid maintenance is 44.0 mL/hr.\n",
        'Answer': 44.0}"
    """

    weight_exp, weight = weight_conversion_explanation(
        input_parameters["weight"]
    )

    explanation = ""

    explanation += weight_exp

    if weight < 10:
        answer = round_number(weight * 4)
        explanation += (
            f"For patient's with weight less than 10 kg, "
            f"the rule for computing maintenance fluid is "
            f"to multiply their weight by 4 mL/kg/hr to "
            f"get the maintenance fluids per hour. Hence, "
            f"the patient's maintenance fluid is {weight} "
            f"kg * 4 mL/kg/hr = {answer} mL/hr.\n"
        )
    elif 10 <= weight <= 20:
        answer = round_number(40 + 2 * (weight - 10))
        explanation += (
            f"For patient's whose weight is in between 10 kg and "
            f"20 kg, the formula for computing maintenance fluid "
            f"is 40 mL/hr + 2 mL/kg/hr * (weight (in kilograms) "
            f"- 10 kilograms). Hence, plugging into this "
            f"formula, we get 40 mL/hr + 2 mL/kg/hr * "
            f"({weight} kg - 10 kg) = {answer} mL/hr.\n"
        )
    elif weight > 20:
        answer = round_number(60 + (weight - 20))
        explanation += (
            f"For patient's whose weight is greater than 20 kg, "
            f"the formula for computing the maintenance fluid is "
            f"60 mL/hr + 1 mL/kg/hr * (weight (in kilograms) "
            f"- 20 kilograms). Hence, plugging into "
            f"this formula, we get 60 mL/hr + 2 mL/kg/hr "
            f"* ({weight} kg - 20 kg) = {answer} mL/hr.\n"
        )

    explanation += (
        f"Hence, the patient's fluid maintenance is {answer} " f"mL/hr.\n"
    )

    return {"Explanation": explanation, "Answer": answer}


if __name__ == "__main__":
    # Defining test cases
    test_cases = [
        {
            "weight": (12.0, "kg"),  # weight 150 lbs
        }
    ]

    # Iterate the test cases and print the results
    for i, input_variables in enumerate(test_cases, 1):
        print(f"Test Case {i}: Input = {input_variables}")
        result = maintenance_fluid_explanation(input_variables)
        print(result)
        print("-" * 50)



# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
r"""
This code is borrowed and modified based on the source code from the
'MedCalc-Bench' repository.
Original repository: https://github.com/ncbi-nlp/MedCalc-Bench

Modifications include:
- rewrite function rautaharju_calculator_explanation
- translation

Date: March 2025
"""

from .utils.rounding import round_number


def rautaharju_calculator_explanation(input_variables):
    r"""
    Calculates the patient's corrected QT interval using the Fridericia
    Formula and generates a detailed explanatory
    text.

    Parameters:
        input_variables (dict): A dictionary containing the following
        key-value pairs:
            - "heart_rate" (tuple): The patient's heart rate in
            the format (value, unit).
                - Value (float): The value of the patient's heart rate.
                - Unit (str): The unit of heart rate should be "beats per
                minute".
            - "qt_interval" (tuple): The QT interval of 330 msec.
                - Value (float): The value of QT interval.
                - Unit (str): The unit of QT interval, "msec".

    Returns:
        dict: Contains two key-value pairs:
            - "Explanation" (str): A detailed description of
            the calculation process.
            - "Answer" (float): The patient's corrected QT interval using
            the Fridericia Formula.

    Notes:
        - None

    Example:
        rautaharju_calculator_explanation({"heart_rate": (110, "beats per
        minute"),"qt_interval": (330, "msec"),})

        output: "{'Explanation': "The corrected QT interval using the
        Rautajarju formula is computed as QTc = QT interval x (120 + HR) /
        180, where QT interval is in msec, and HR is the heart rate in
        beats per minute. \nThe QT interval is 330 msec.\nThe patient's
        heart rate is 110 beats per minute.\nHence, plugging in these
        values, we will get 330 x (120 + 110) / 180 = 421.667.\nThe
        patient's corrected QT interval (QTc) is 421.667 msec.\n",
        'Answer': 421.667}"
    """

    heart_rate = input_variables["heart_rate"][0]
    qt_interval = input_variables["qt_interval"][0]

    explanation = (
        "The corrected QT interval using the Rautajarju formula is computed "
        "as QTc = QT interval x (120 + HR) / 180, where QT interval is in "
        "msec, and HR is the heart rate in beats per minute.\n"
    )

    explanation += f"The QT interval is {qt_interval} msec.\n"
    explanation += (
        f"The patient's heart rate is {heart_rate} beats per minute.\n"
    )

    qt_c = round_number(qt_interval * (120 + heart_rate) / 180)

    explanation += (
        f"Hence, plugging in these values, we will get "
        f"{qt_interval} x (120 + {heart_rate}) / 180 = {qt_c}.\n"
    )
    explanation += (
        f"The patient's corrected QT interval (QTc) is {qt_c} msec.\n"
    )

    return {"Explanation": explanation, "Answer": qt_c}


if __name__ == "__main__":
    # Defining test cases
    test_cases = [
        {
            "heart_rate": (110, "beats per minute"),
            "qt_interval": (330, "msec"),
        },
        {
            "heart_rate": (57, "beats per minute"),
            "qt_interval": (330, "msec"),
        },
    ]

    # Iterate the test cases and print the results
    for i, input_variables in enumerate(test_cases, 1):
        print(f"Test Case {i}: Input = {input_variables}")
        result = rautaharju_calculator_explanation(input_variables)
        print(result)
        print("-" * 50)



# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
r"""
This code is borrowed and modified based on the source code from the
'MedCalc-Bench' repository.
Original repository: https://github.com/ncbi-nlp/MedCalc-Bench

Modifications include:
- rewrite function compute_perc_rule_explanation
- translation

Date: March 2025
"""

from .utils.age_conversion import (
    age_conversion_explanation,
)


def compute_perc_rule_explanation(input_parameters):
    perc_count = 0

    explanation = r"""
    The PERC Rule critiera are listed below:
    
       1. Age ≥50: No = 0 points, Yes = +1 point
       2. Heart Rate (HR) ≥100: No = 0 points, Yes = +1 point
       3. O₂ saturation on room air <95%: No = 0 points, Yes = +1 point
       4. Unilateral leg swelling: No = 0 points, Yes = +1 point
       5. Hemoptysis: No = 0 points, Yes = +1 point
       6. Recent surgery or trauma (within 4 weeks, requiring treatment 
       with general anesthesia): No = 0 points, Yes = +1 point
       7. Prior pulmonary embolism (PE) or deep vein thrombosis (DVT): No = 
       0 points, Yes = +1 point
       8. Hormone use (oral contraceptives, hormone replacement, 
       or estrogenic hormone use in males or females): No = 0 points, 
       Yes = +1 point
    
    The total number of criteria met is taken by summing 
    the points for each criterion.\n\n
    """

    explanation += "The current count of PERC criteria met is 0.\n"

    age_exp, age = age_conversion_explanation(input_parameters["age"])
    heart_rate = input_parameters["heart_rate"][0]
    oxygen_sat = input_parameters["oxygen_sat"][0]

    parameters = {
        "unilateral_leg_swelling": "unilateral leg swelling",
        "hemoptysis": "hemoptysis",
        "recent_surgery_or_trauma": "recent surgery or trauma",
        "previous_pe": "prior pulmonary embolism",
        "previous_dvt": "prior deep vein thrombosis",
        "hormonal_use": "hormonal use",
    }

    explanation += age_exp
    if age >= 50:
        explanation += (
            f"The patient's age is greater than or equal to 50 "
            f"years, and so we increment the perc critieria met "
            f"by 1, making the current total {perc_count} + 1 = "
            f"{perc_count + 1}.\n"
        )
        perc_count += 1
    else:
        explanation += (
            f"The patient's age is less than 50 years, and so we "
            f"do not increment the criteria count. The current "
            f"total remains at {perc_count}.\n"
        )

    explanation += (
        f"The patient's heart rate is {heart_rate} beats per " f"minute. "
    )

    if heart_rate >= 100:
        explanation += (
            f"The patient's heart rate is greater than or equal "
            f"to 100 beats per minute, and so we increment the "
            f"perc critieria met by 1, making the current total "
            f"{perc_count} + 1 = {perc_count + 1}.\n"
        )
        perc_count += 1
    else:
        explanation += (
            f"The patient's heart rate is less than 100 beats "
            f"per minute, and so we do not increment the "
            f"criteria count. The current total "
            f"remains at {perc_count}.\n"
        )

    explanation += (
        f"The saturated oxygen percentage in the room is "
        f"{oxygen_sat} percent. "
    )

    if oxygen_sat < 95:
        explanation += (
            f"The saturated oxygen is less than 95%, and so we "
            f"increment the perc critieria met by 1, making the "
            f"current total {perc_count} + 1 = {perc_count + 1}.\n"
        )
        perc_count += 1
    else:
        explanation += (
            f"The saturated oxygen is greater than or equal to "
            f"95% and so we do not increment the criteria count. "
            f"The current total remains at {perc_count}.\n"
        )

    for parameter in parameters:
        if parameter == "previous_pe":
            continue

        if parameter == "previous_dvt":
            explanation += (
                "The patient must be diagnosed with at least one "
                "of deep vein thrombosis or pulmonary embolism "
                "in the past for a PERC rule criteria to be met. "
            )

            if 'previous_dvt' not in input_parameters:
                explanation += (
                    "Whether the patient has been diagnosed for "
                    "pulmonary embolism in the past is not "
                    "reported. Hence, we assume it to be absent. "
                )
                input_parameters['previous_dvt'] = False
            elif not input_parameters['previous_dvt']:
                explanation += (
                    "The patient is not reported to have been "
                    "diagnosed with deep vein thrombosis in the "
                    "past. "
                )
            else:
                explanation += (
                    "The patient is reported to have been "
                    "diagnosed with deep vein thrombosis in the "
                    "past. "
                )

            if 'previous_pe' not in input_parameters:
                explanation += (
                    "Whether the patient has been diagnosed for "
                    "pulmonary embolism in the past is not "
                    "reported. Hence, we assume it to be absent. "
                )
                input_parameters['previous_pe'] = False
            elif not input_parameters['previous_pe']:
                explanation += (
                    "The patient is not reported to have been "
                    "diagnosed with pulmonary embolism in the "
                    "past. "
                )
            else:
                explanation += (
                    "The patient is reported to have been "
                    "diagnosed with pulmonary embolism in the "
                    "past. "
                )

            if (
                input_parameters['previous_dvt']
                or input_parameters['previous_pe']
            ):
                explanation += (
                    f"At least one of the criteria is met and so "
                    f"we increment the criteria met by 1, making "
                    f"the current total {perc_count} + 1 = "
                    f"{perc_count + 1}.\n"
                )
                perc_count += 1
            else:
                explanation += (
                    f"Neither criteria is met and so we do "
                    f"increment the criteria count, keep the "
                    f"current total at {perc_count}.\n"
                )
            continue

        if parameter not in input_parameters:
            explanation += (
                f"The patient note does not report a status on '"
                f"{parameters[parameter]}'. Hence, we assume it "
                f"to be absent, and so we do not increment the "
                f"criteria count. The current total remains at "
                f"{perc_count}.\n"
            )
        elif not input_parameters[parameter]:
            explanation += (
                f"The patient note reports '"
                f"{parameters[parameter]}' to be absent in the "
                f"patient and so we do not increment "
                f"the criteria count. The current total remains "
                f"at {perc_count}.\n"
            )
        else:
            explanation += (
                f"The patient note reports '"
                f"{parameters[parameter]}' to be present for the "
                f"patient and so we increment the criteria count "
                f"by 1, making the current total "
                f"{perc_count} + 1  =  {perc_count + 1}.\n"
            )
            perc_count += 1

    explanation += (
        f"Hence, the number of PERC rule criteria met by the "
        f"patient is {perc_count}.\n"
    )

    return {"Explanation": explanation, "Answer": perc_count}


if __name__ == "__main__":
    # Defining test cases
    test_cases = [
        {
            "age": (73, "years"),
            "heart_rate": (92.0, "breaths per minute"),
            "oxygen_sat": (98.0, '%'),
            "unilateral_leg_swelling": False,
            "hemoptysis": False,
            "recent_surgery_or_trauma": True,
            "previous_pe": False,
            "previous_dvt": False,
            "hormonal_use": True,
        }
    ]

    # Iterate the test cases and print the results
    for i, input_variables in enumerate(test_cases, 1):
        print(f"Test Case {i}: Input = {input_variables}")
        result = compute_perc_rule_explanation(input_variables)
        print(result)
        print("-" * 50)



# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
r"""
This code is borrowed and modified based on the source code from the
'MedCalc-Bench' repository.
Original repository: https://github.com/ncbi-nlp/MedCalc-Bench

Modifications include:
- rewrite function compute_meldna_explanation
- translation

Date: March 2025
"""

import math

from .utils.unit_converter_new import (
    conversion_explanation,
)


def compute_meldna_explanation(input_variables):
    r"""
    Calculates the patient's child pugh score and generates a detailed
    explanatory text.

    Parameters:
        input_variables (dict): A dictionary containing the following
        key-value pairs:
            - "creatinine" (tuple): The patient's creatinine information in the
            format (value, unit).
                - Value (float): The value of creatinine.
                - Unit (str): The unit of creatinine,
                which can be "mg/dL", "μmol/L", and so on.
            - "dialysis_twice" (boolean):
            - "inr" (float): The patient's international normalised ratio (
            INR) in the float format.
            - "bilirubin" (array): The patient's bilirubin level in the
            format (value, unit).
                - Value (float): The value of bilirubin level.
                - Unit (str): The unit of bilirubin level, eg. "mmol/L",
                "mEq/L", and so on.
            - "sodium" (array): The patient's bicarbonate level in the
            format (value, unit).
                - Value (float): The value of bicarbonate level.
                - Unit (str): The unit of bicarbonate level, eg. "mmol/L",
                "mEq/L", and so on.
            - "creatinine" (tuple): The patient's creatinine information in the
            format (value, unit).
                - Value (float): The value of creatinine.
                - Unit (str): The unit of creatinine,
                which can be "mg/dL", "μmol/L", and so on.

    Returns:
        dict: Contains two key-value pairs:
            - "Explanation" (str): A detailed description of
            the calculation process.
            - "Answer" (float): The patient's MELD-Na score.

    Notes:
        - None

    Example:
        compute_meldna_explanation({'creatinine': (1.0, 'mg/dL'),
        'dialysis_twice': False,
        'bilirubin': (2.8, 'mg/dL'),
        'inr': 1.5, 'sodium': [139.0, 'mEq/L']})

        output: "{'Explanation': "The formula for computing the MELD Na is
        to first apply the following equation: MELD(i) = 0.957 x ln(Cr) +
        0.378 x ln(bilirubin) + 1.120 x ln(INR) + 0.643.\nIf the MELD(i) is
        greater than 11 after rounding to the nearest tenth and multiplying
        the MELD(i) by 10, we apply the following equation: MELD = MELD(i)
        + 1.32 x (137 - Na) -  [ 0.033 x MELD(i) x (137 - Na)]. The MELD Na
        score is capped at 40. The concentration of Na is mEq/L,
        the concentration of bilirubin is mg/dL, and the concentration of
        creatinine is mg/dL. If the patient's Na concentration is less than
        125 mEq/L, we set it to 125 mEq/L and if the patient's the Na
        concentration is greater 137 mEq/L, we round it to 137 mEq/L.\nThe
        concentration of creatinine is 1.0 mg/dL. \nThe patient has not went
        through dialysis at least twice in the past week.\nWhether the
        patient has gone through continuous veno-venous hemodialysis in the
        past 24 hours is not mentioned, and so we assume this to be false.
        \nThe concentration of bilirubin is 2.8 mg/dL. \nThe patient's INR
        is 1.5. \nThe concentration of sodium is 139.0 mEq/L. The sodium
        concentration is greater than 137 mEq/L, and so we set the sodium
        concentration to 137 mEq/L.\nApplying the first equation givesus
        0.957 x ln(1.0) + 0.378 x ln(2.8) + 1.120 x ln(1.5) + 0.643 =
        1.486317060775622. Rounding to the nearest tenth makesthe MELD (i)
        score 1.5. We then multiply by 10, making the MELD(i) score 15.
        \nBecause the MELD (i) score is greater than 11, we then apply the
        second equation, giving us 15 + 1.32 x (137 - 137) -  [0.033 x
        1.486317060775622 x (137 - 137)] = 15.\nThe MELD Na score is less
        than 40, and so we keep the score as it is. The patient's MELDNa
        score, rounded to the nearest integer, is 15 points.\n", 'Answer': 15}"
    """

    explanation = (
        "The formula for computing the MELD Na is to first apply "
        "the following equation: MELD(i) = 0.957 x ln(Cr) + 0.378 "
        "x ln(bilirubin) + 1.120 x ln(INR) + 0.643.\n"
    )
    explanation += (
        "If the MELD(i) is greater than 11 after rounding to the "
        "nearest tenth and multiplying the MELD(i) by 10, "
        "we apply the following equation: MELD = MELD(i) + "
        "1.32 x (137 - Na) -  [ 0.033 x MELD(i) x (137 - Na)]. "
        "The MELD Na score is capped at 40. "
    )
    explanation += (
        "The concentration of Na is mEq/L, the concentration of "
        "bilirubin is mg/dL, and the concentration of creatinine "
        "is mg/dL. If the patient's Na concentration is less "
        "than 125 mEq/L, we set it to 125 mEq/L and if the "
        "patient's the Na concentration is greater 137 mEq/L, "
        "we round it to 137 mEq/L.\n"
    )

    creatinine_exp, creatinine = conversion_explanation(
        input_variables["creatinine"][0],
        "creatinine",
        113.12,
        None,
        input_variables["creatinine"][1],
        "mg/dL",
    )

    explanation += creatinine_exp + "\n"

    if "dialysis_twice" not in input_variables:
        explanation += (
            "Whether the patient has gone through dialysis at "
            "least twice in the past week is not mentioned, "
            "and so we assume this to be false.\n"
        )
        input_variables["dialysis_twice"] = False
    elif input_variables["dialysis_twice"]:
        explanation += (
            "The patient is reported to have went through "
            "dialysis at least twice in the past week.\n"
        )
    else:
        explanation += (
            "The patient has not went through dialysis at least "
            "twice in the past week.\n"
        )

    if "cvvhd" not in input_variables:
        explanation += (
            "Whether the patient has gone through continuous "
            "veno-venous hemodialysis in the past 24 hours "
            "is not mentioned, and so we assume this to be false.\n"
        )
        input_variables["cvvhd"] = False
    elif input_variables["cvvhd"]:
        explanation += (
            "The patient is reported to have went through "
            "continuous veno-venous hemodialysis in the past 24 "
            "hours.\n"
        )
    else:
        explanation += (
            "The patient is reported to not have done "
            "dialysis at least twice in the past week.\n"
        )

    if creatinine < 1.0:
        explanation += (
            "The patient's creatinine concentration is less than "
            "1.0 mg/dL, and so we set the creatinine "
            "concentration to 1.0 mg/dL.\n"
        )
        creatinine = 1.0
    elif creatinine > 4.0:
        explanation += (
            "The creatinine concentration is greater than 4.0 "
            "mg/dL, and so we set the creatinine "
            "concentration to 4.0 mg/dL.\n"
        )
        creatinine = 4.0
    elif input_variables["dialysis_twice"] or input_variables["cvvhd"]:
        explanation += (
            "Because the patient has gone through at least one "
            "of (i) dialysis two or more times in the past 7 "
            "days or (ii) continuous veno-venous hemodialysis "
            "in the past 24 hours, we set the creatinine "
            "concentration to 4.0 mg/dL.\n"
        )
        creatinine = 4.0

    bilirubin_exp, bilirubin = conversion_explanation(
        input_variables["bilirubin"][0],
        "bilirubin",
        None,
        None,
        input_variables["bilirubin"][1],
        "mg/dL",
    )

    explanation += bilirubin_exp

    if bilirubin < 1.0:
        explanation += (
            "The patient's bilirubin concentration is less than "
            "1.0 mg/dL, and so we set the bilirubin concentration "
            "to 1.0 mg/dL.\n"
        )
        bilirubin = 1.0
    else:
        explanation += "\n"

    inr = input_variables["inr"]

    explanation += f"The patient's INR is {inr}. "

    if inr < 1.0:
        explanation += (
            "The patient's INR is less than 1.0, and so we set "
            "the INR to 1.0.\n"
        )
        inr = 1.0
    else:
        explanation += "\n"

    sodium_exp, sodium = conversion_explanation(
        input_variables["sodium"][0],
        "sodium",
        22.99,
        1,
        input_variables["sodium"][1],
        "mEq/L",
    )

    explanation += sodium_exp

    if sodium < 125:
        explanation += (
            "The sodium concentration is less than 125 mEq/L, "
            "and so we set the sodium concentration to 125 mEq/L.\n"
        )
        sodium = 125
    elif sodium > 137:
        explanation += (
            "The sodium concentration is greater than 137 mEq/L, "
            "and so we set the sodium concentration to 137 mEq/L.\n"
        )
        sodium = 137
    else:
        explanation += "\n"

    meld_i = (
        0.957 * math.log(creatinine)
        + 0.378 * math.log(bilirubin)
        + 1.120 * math.log(inr)
        + 0.643
    )
    meld_i_rounded = round(meld_i, 1)
    meld_10 = round(meld_i_rounded * 10)

    explanation += (
        f"Applying the first equation gives"
        f"us 0.957 x ln({creatinine}) + 0.378 x "
        f"ln({bilirubin}) + 1.120 x ln({inr}) + 0.643 = {meld_i}. "
    )
    explanation += (
        f"Rounding to the nearest tenth makes"
        f"the MELD (i) score {meld_i_rounded}. We then multiply "
        f"by 10, making the MELD(i) score {meld_10}.\n"
    )

    meld = round(
        meld_10 + 1.32 * (137 - sodium) - (0.033 * meld_10 * (137 - sodium))
    )

    if meld_10 > 11:
        explanation += (
            f"Because the MELD (i) score is greater than 11, "
            f"we then apply the second equation, giving us "
            f"{meld_10} + 1.32 x (137 - {sodium}) -  [0.033 x "
            f"{meld_i} x (137 - {sodium})] = {meld}.\n"
        )

        if meld > 40:
            meldna = 40
            explanation += (
                "The maximum the MELD Na score can be is 40, "
                "and so the patient's MELD score is 40."
            )
        else:
            meldna = meld
            explanation += (
                f"The MELD Na score is less than 40, and so we "
                f"keep the score as it is. The patient's MELDNa "
                f"score, rounded to the nearest integer, "
                f"is {round(meldna)} points.\n"
            )

    else:
        meldna = meld_10
        explanation += (
            f"The patient's MELD (i) score is less than 11, "
            f"and so we do not apply the second equation, making "
            f"the patient's MELD Na score, {round(meldna)} "
            f"points.\n"
        )

    return {"Explanation": explanation, "Answer": round(meldna)}


if __name__ == "__main__":
    # Defining test cases
    test_cases = [
        {
            "creatinine": (1.0, "mg/dL"),
            "dialysis_twice": False,
            'bilirubin': (2.8, "mg/dL"),
            "inr": 1.5,
            'sodium': [139.0, 'mEq/L'],
        }
    ]

    # Iterate the test cases and print the results
    for i, input_variables in enumerate(test_cases, 1):
        print(f"Test Case {i}: Input = {input_variables}")
        result = compute_meldna_explanation(input_variables)
        print(result)
        print("-" * 50)



# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
r"""
This code is borrowed and modified based on the source code from the
'MedCalc-Bench' repository.
Original repository: https://github.com/ncbi-nlp/MedCalc-Bench

Modifications include:
- rewrite function compute_cci_explanation
- translation

Date: March 2025
"""

from .utils.age_conversion import (
    age_conversion_explanation,
)


def generate_cha2ds2_vasc_explanation(params):
    score = 0

    output = """
    The criteria for the CHA2DS2-VASc score are listed below:

    1. Age: < 65 years = 0 points, 65-74 years = +1 point, ≥ 75 years
        = +2 points
    2. Sex: Female = +1 point, Male = 0 points
    3. Congestive Heart Failure (CHF) history: No = 0 points, Yes = +1 point
    4. Hypertension history: No = 0 points, Yes = +1 point
    5. Stroke, Transient Ischemic Attack (TIA), or Thromboembolism history:
        No = 0 points, Yes = +2 points
    6. Vascular disease history (previous myocardial infarction, peripheral
        artery disease, or aortic plaque): No = 0 points, Yes = +1 point
    7. Diabetes history: No = 0 points, Yes = +1 point

    The CHA2DS2-VASc score is calculated by summing the points for each
        criterion.\n\n
    """

    output += "The current CHA2DS2-VASc score is 0.\n"

    text, age = age_conversion_explanation(params['age'])
    output += text

    # Age
    if age >= 75:
        output += (
            f"Because the age is greater than 74, two points added to "
            f"the score, making the current total {score} + 2 = "
            f"{score + 2}.\n"
        )
        score += 2
    elif age >= 65:
        output += (
            f"Because the age is between 65 and 74, one point added "
            f"to the score, making the current total {score} + 1 = "
            f"{score + 1}.\n"
        )
        score += 1
    else:
        output += (
            f"Because the age is less than 65 years, no points are "
            f"added to the current total, keeping the total at "
            f"{score}.\n"
        )

    sex = params['sex']  # Sex of the patient (Male/Female)

    output += f"The patient's gender is {sex.lower()} "

    if sex.lower() == 'female':
        output += (
            f"and so one point is added to the score, making the "
            f"current total {score} + 1 = {score + 1}.\n"
        )
        score += 1
    else:
        output += (
            f"and so no points are added to the current total, "
            f"keeping the total at {score}.\n"
        )

    # Congestive Heart Failure
    if 'chf' in params:
        chf = params['chf']
        output += (
            f"The patient history for congestive heart failure is "
            f"{'present' if chf else 'absent'}. "
        )
    else:
        chf = False
        output += (
            "Because the congestive heart failure history is not "
            "specified in the patient note, we assume it is absent "
            "from the patient. "
        )

    # Congestive Heart Failure (CHF)
    if chf:
        output += (
            f"Because the patient has congestive heart failure, "
            f"one point is added to the score, making the current "
            f"total {score} + 1 = {score + 1}.\n"
        )
        score += 1
    else:
        output += (
            f"Because the patient does not have congestive heart "
            f"failure, no points are added to the current total, "
            f"keeping the total at {score}.\n"
        )

    # Hypertension
    if 'hypertension' in params:
        hypertension = params['hypertension']
        output += (
            f"The patient history for hypertension is "
            f"{'present' if hypertension else 'absent'}. "
        )
    else:
        hypertension = False
        output += (
            "Because hypertension history is not specified in the "
            "patient note, we assume that it is absent from the "
            "patient. "
        )

    # Congestive Heart Failure (CHF)
    if hypertension:
        output += (
            f"Because the patient has hypertension, one point is "
            f"added to the score, making the current "
            f"total {score} + 1 = {score + 1}.\n"
        )
        score += 1
    else:
        output += (
            f"Because the patient does not have hypertension, "
            f"no points are added to the current total, "
            f"keeping the total at {score}.\n"
        )

    output += (
        "One criteria of the CHA2DS2-VASc score is to check "
        "if the patient has had any history of stroke, transient "
        "ischemic attacks (TIA), or thromboembolism. "
    )

    if 'stroke' in params:
        stroke = params['stroke']
        output += (
            f"Based on the patient note, the patient history for "
            f"stroke is {'present' if stroke else 'absent'}. "
        )
    else:
        stroke = False
        output += (
            "Because stroke history is not specified in the patient "
            "note, we assume that it is absent from the patient. "
        )

    if 'tia' in params:
        tia = params['tia']
        output += (
            f"Based on the patient note, the patient history for tia "
            f"is {'present' if tia else 'absent'}. "
        )
    else:
        tia = False
        output += (
            "Because tia history is not specified in the patient "
            "note, we assume that it is absent from the patient. "
        )

    if 'thromboembolism' in params:
        thromboembolism = params['thromboembolism']
        output += (
            f"Based on the patient note, the patient history for "
            f"thromboembolism is "
            f"{'present' if thromboembolism else 'absent'}. "
        )
    else:
        thromboembolism = False
        output += (
            "Because thromboembolism history is not specified in the "
            "patient note, we assume it to be absent. "
        )

    # Stroke / TIA / Thromboembolism
    if stroke or tia or thromboembolism:
        output += (
            f"Because at least one of stroke, tia, or thromboembolism "
            f"is present, two points are added to the score, making "
            f"the current total {score} + 2 = {score + 2}.\n"
        )
        score += 2
    else:
        output += (
            f"Because all of stroke, tia, or thromboembolism are "
            f"absent, no points are added to score, keeping the score "
            f"at {score}.\n"
        )

    if 'vascular_disease' in params:
        vascular_disease = params['vascular_disease']
        output += (
            f"Based on the patient note, the patient history for "
            f"vascular disease is "
            f"{'present' if vascular_disease else 'absent'}. "
        )
    else:
        vascular_disease = False
        output += (
            "Because vascular disease history is not specified "
            "in the patient note, we assume it to be absent.\n"
        )

    if vascular_disease:
        output += (
            f"Because the patient has vascular disease, one point is "
            f"added to the score, making the current "
            f"total {score} + 1 = {score + 1}. "
        )
        score += 1
    else:
        output += (
            f"Because the patient does not have vascular disease, "
            f"no points are added to score, keeping the score at "
            f"{score}. "
        )

    if 'diabetes' in params:
        diabetes = params['diabetes']
        output += (
            f"Based on the patient note, the patient history for "
            f"diabetes is {'present' if diabetes else 'absent'}. "
        )
    else:
        diabetes = False
        output += (
            "Because diabetes history is not specified in the "
            "patient note, we assume it's value as 'absent'. "
        )

    if diabetes:
        output += (
            f"Because the patient has diabetes, one point "
            f"is added to the score, making the current total {score} "
            f"+ 1 = {score + 1}.\n"
        )
        score += 1
    else:
        output += (
            f"Because the patient does not have diabetes, "
            f"no points are added to score, keeping the score at "
            f"{score}.\n"
        )

    output += f"The patient's CHA2DS2-VASc Score is {score}.\n"

    return {"Explanation": output, "Answer": score}


if __name__ == "__main__":
    # Defining test cases
    test_cases = [
        {
            'sex': 'Male',
            'thromboembolism': True,
            'tia': True,
            'hypertension': True,
            'age': (78, 'years'),
            'stroke': True,
        }
    ]

    # Iterate the test cases and print the results
    for i, input_variables in enumerate(test_cases, 1):
        print(f"Test Case {i}: Input = {input_variables}")
        result = generate_cha2ds2_vasc_explanation(input_variables)
        print(result)
        print("-" * 50)



# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
r"""
This code is borrowed and modified based on the source code from the
'MedCalc-Bench' repository.
Original repository: https://github.com/ncbi-nlp/MedCalc-Bench

Modifications include:
- rewrite function calculate_corrected_calcium_explanation
- translation

Date: March 2025
"""

from .utils.rounding import round_number
from .utils.unit_converter_new import (
    conversion_explanation,
)


def calculate_corrected_calcium_explanation(params):
    r"""
    Calculates the patient's corrected calcium concentration and
    generates a detailed explanatory text.

    Parameters:
        params (dict): A dictionary containing the following key-value pairs:
            - "albumin" (array): The patient's albumin concentration in the
            format (value, unit).
                - Value (float): The numerical albumin concentration value.
                - Unit (str): The unit of albumin concentration, eg. "g/L",
                "mg/dL", "g/mL" and so on.
            - "calcium" (array): The patient's calcium concentration in the
            format (value, unit).
                - Value (float): The numerical calcium concentration value.
                - Unit (str): The unit of calcium concentration, eg. "g/L",
                "mg/dL", "g/mL" and so on.

    Returns:
        dict: Contains two key-value pairs:
            - "Explanation" (str): A detailed description of
            the calculation process.
            - "Answer" (float): The patient's corrected calcium
            concentration (in mg/dL).

    Notes:
        - Uses the `conversion_explanation` function to convert Albumin
        level to standard unit g/dL.
        - Uses the `conversion_explanation` function to convert Calcium
        level to standard unit mg/dL.

    Example:
        calculate_corrected_calcium_explanation({"albumin": [4, "mg/dL"],
        "calcium": [40, "mg/L"]})

        output: "{'Explanation': "To compute the patient's correct calcium
        level in mg/dL, the formula is  (0.8 * (Normal Albumin (in g/dL) -
        Patient's Albumin (in g/dL))) + Serum Calcium (in mg/dL). \nThe
        patient's normal albumin level is 4.0 g/dL.\nThe concentration of
        Albmumin is 4 g/dL. \nThe concentration of Calcium is 40 mg/L. We
        need to convert the concentration to mg/dL. The mass units of the
        source and target are the same so no conversion is needed. The
        current volume unit is L and the target volume unit is dL. The
        conversion factor is 10.0 dL for every unit of L.
        Our next step will be to divide the mass by the volume conversion
        factor of 10.0 to get the final concentration in terms of mg/dL.
        This will result to 40 mg Calcium/10.0 dL = 4.0 mg Calcium/dL. The
        concentration value of 40 mg Calcium/L converts to 4.0 mg
        Calcium/dL. \nPlugging these values into the formula, we get (0.8 *
        (4.0 g/dL - 4 g/dL)) + 4.0 mg/dL = 4.0 mg/dL. \nThe patient's
        corrected calcium concentration 4.0 mg/dL.\n", 'Answer': 4.0}"
    """

    # Extract parameters from the input dictionary
    normal_albumin = 4.0  # Normal albumin level in g/dL

    albumin = params.get('albumin')
    albumin_val = albumin[0]
    albumin_units = albumin[1]

    calcium = params.get('calcium')
    calcium_val = calcium[0]
    calcium_units = calcium[1]

    output = (
        "To compute the patient's correct calcium level in mg/dL, "
        "the formula is "
        "(0.8 * (Normal Albumin (in g/dL) - Patient's Albumin (in g/dL))) + "
        "Serum Calcium (in mg/dL).\n"
    )

    # Generate explanation
    output += "The patient's normal albumin level is 4.0 g/dL.\n"
    albumin_explanation, albumin = conversion_explanation(
        albumin_val, "Albmumin", 66500, None, albumin_units, "g/dL"
    )
    calcium_explanation, calcium = conversion_explanation(
        calcium_val, "Calcium", 40.08, 2, calcium_units, "mg/dL"
    )

    output += f"{albumin_explanation}\n"
    output += f"{calcium_explanation}\n"

    corrected_calcium = round_number(
        0.8 * (normal_albumin - albumin) + calcium
    )

    output += "Plugging these values into the formula, we get "
    output += (
        f"(0.8 * ({normal_albumin} g/dL - {albumin} g/dL)) + "
        f"{calcium} mg/dL = {corrected_calcium} mg/dL.\n"
    )

    output += (
        f"The patient's corrected calcium concentration "
        f"{corrected_calcium} mg/dL.\n"
    )

    return {"Explanation": output, "Answer": corrected_calcium}


if __name__ == "__main__":
    # Defining test cases
    test_cases = [
        {
            "albumin": (4, "g/dL"),
            "calcium": (40, "mg/L"),
        }
    ]

    # Iterate the test cases and print the results
    for i, input_variables in enumerate(test_cases, 1):
        print(f"Test Case {i}: Input = {input_variables}")
        result = calculate_corrected_calcium_explanation(input_variables)
        print(result)
        print("-" * 50)



# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
r"""
This code is borrowed and modified based on the source code from the
'MedCalc-Bench' repository.
Original repository: https://github.com/ncbi-nlp/MedCalc-Bench

Modifications include:
- rewrite function compute_serum_osmolality_explanation
- translation

Date: March 2025
"""

from .utils.rounding import round_number
from .utils.unit_converter_new import (
    conversion_explanation,
)


def compute_serum_osmolality_explanation(input_parameters):
    r"""
    Calculates the patient's Serum Osmolality and generates a detailed
    explanatory text.

    Parameters:
        input_parameters (dict): A dictionary containing the following
        key-value pairs:
            - "bun" (array): The patient's blood urea nitrogen level in the
            format (value, unit).
                - Value (float): The value of blood urea nitrogen level.
                - Unit (str): The unit of blood urea nitrogen level,
                eg. "mg/dL", "mEq/L", and so on.
            - "glucose" (array): The patient's blood glucose level in the
            format (value, unit).
                - Value (float): The value of blood glucose level.
                - Unit (str): The unit of blood glucose level,
                eg. "mmol/L", "mEq/L", and so on.
            - "sodium" (array): The patient's bicarbonate level in the
            format (value, unit).
                - Value (float): The value of bicarbonate level.
                - Unit (str): The unit of bicarbonate level, eg. "mmol/L",
                "mEq/L", and so on.

    Returns:
        dict: Contains two key-value pairs:
            - "Explanation" (str): A detailed description of
            the calculation process.
            - "Answer" (float): The patient's Serum Osmolality.

    Notes:
        - None

    Example:
        compute_serum_osmolality_explanation({'bun': [20.0, 'mg/dL'],
        'glucose': [599.0, 'mg/dL'],'sodium': [139.0, 'mEq/L']})

        output: "{'Explanation': "The formula for computing serum osmolality
        is 2 * Na + (BUN / 2.8) + (glucose / 18), where Na is the
        concentration of sodium in mmol/L, the concentration of BUN is in
        mg/dL, and the concentration of glucose is in mg/dL.\nThe
        concentration of sodium is 139.0 mEq/L. We need to convert the
        concentration to mmol/L. Let's first convert the mass of sodium
        from mEq to mmol. The mass of sodium is 139.0 mEq. To convert from
        139.0 mEq to mmol, convert from mEq to mmol. The compound 139.0 has
        a valence of 1, and so divide the valence by the value of mEq to
        get, 139.0 mEq/(1 mEq/mmol) = 139.0 mmol sodium. The volume units
        is L so no volume conversion is needed. Hence, the concentration
        value of 139.0 mEq sodium/L converts to 139.0 mmol sodium/L. \nThe
        concentration of bun is 20.0 mg/dL. \nThe concentration of glucose
        is 599.0 mg/dL. \nPlugging these values into the equation, we get 2
        * 139.0 + (20.0 / 2.8) + (20.0 / 18) = 318.421 mmol/L.
        The patient's calculated serum osmolality concentration is 318.421
        mmol/L. This is equalivalent to 318.421 mOsm/kg.\n", 'Answer':
        318.421}"
    """

    explanation = (
        "The formula for computing serum osmolality is 2 * Na + (BUN / 2.8) "
        "+ (glucose / 18), where Na is the concentration of sodium in "
        "mmol/L, the concentration of BUN is in mg/dL, "
        "and the concentration of glucose is in mg/dL.\n"
    )

    sodium_exp, sodium = conversion_explanation(
        input_parameters["sodium"][0],
        "sodium",
        22.99,
        1,
        input_parameters["sodium"][1],
        "mmol/L",
    )
    bun_exp, bun = conversion_explanation(
        input_parameters["bun"][0],
        "bun",
        28.02,
        None,
        input_parameters["bun"][1],
        "mg/dL",
    )
    glucose_exp, glucose = conversion_explanation(
        input_parameters["glucose"][0],
        "glucose",
        180.16,
        None,
        input_parameters["glucose"][1],
        "mg/dL",
    )

    explanation += sodium_exp + "\n"
    explanation += bun_exp + "\n"
    explanation += glucose_exp + "\n"

    serum_os = round_number(2 * sodium + (bun / 2.8) + (glucose / 18))

    explanation += (
        f"Plugging these values into the equation, "
        f"we get 2 * {sodium} + ({bun} / 2.8) + ({bun} / 18) = {serum_os} "
        f"mmol/L."
    )
    explanation += (
        f"The patient's calculated serum osmolality concentration is "
        f"{serum_os} mmol/L. This is equalivalent to {serum_os} mOsm/kg.\n"
    )

    return {"Explanation": explanation, "Answer": serum_os}


if __name__ == "__main__":
    # Defining test cases
    test_cases = [
        {
            'bun': [20.0, 'mg/dL'],
            'glucose': [599.0, 'mg/dL'],
            'sodium': [139.0, 'mEq/L'],
        }
    ]

    # Iterate the test cases and print the results
    for i, input_variables in enumerate(test_cases, 1):
        print(f"Test Case {i}: Input = {input_variables}")
        result = compute_serum_osmolality_explanation(input_variables)
        print(result)
        print("-" * 50)



# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
r"""
This code is borrowed and modified based on the source code from the
'MedCalc-Bench' repository.
Original repository: https://github.com/ncbi-nlp/MedCalc-Bench

Modifications include:
- rewrite function compute_ldl_explanation
- translation

Date: March 2025
"""

from .utils.rounding import round_number
from .utils.unit_converter_new import (
    conversion_explanation,
)


def compute_ldl_explanation(input_parameters):
    r"""
    Calculates the patient's LDL cholestrol concentration and generates a
    detailed explanatory text.

    Parameters:
        input_parameters (dict): A dictionary containing the following
        key-value pairs:
            - "hdl_cholestrol" (tuple): The concentration of HDL cholestrol
            in the format (value, unit).
                - Value (float): The value of HDL cholestrol.
                - Unit (str): The unit of HDL cholestrol, eg. "µIU/mL",
                "pmol/L", and so on.
            - "triglycerides" (tuple): The concentration of triglycerides
            in the format (value, unit).
                - Value (float): The concentration of triglycerides.
                - Unit (str): The unit of triglycerides, eg. "mmol/L",
                "mEq/L", and so on.
            - "total_cholestrol" (tuple): The concentration of total
            cholestrol in the format (value, unit).
                - Value (float): The value of total cholestrol.
                - Unit (str): The unit of total cholestrol, eg. "mmol/L",
                "mEq/L", and so on.

    Returns:
        dict: Contains two key-value pairs:
            - "Explanation" (str): A detailed description of
            the calculation process.
            - "Answer" (float): The patient's LDL cholestrol concentration.

    Notes:
        - None

    Example:
        compute_homa_ir_explanation({"hdl_cholestrol": (37.0, 'mg/dL'),
        "triglycerides": (205.0, 'mg/dL'),
        "total_cholestrol": (210.0, 'mg/dL')})

        output: "{'Explanation': "To compute the patient's LDL cholestrol,
        apply the following formula: LDL cholesterol = total cholesterol -
        HDL - (triglycerides / 5), where the units for total cholestrol,
        HDL cholestrol, and triglycerides are all mg/dL.\nThe concentration
        of total cholestrol is 210.0 mg/dL. \nThe concentration of hdl
        cholestrol is 37.0 mg/dL. \nThe concentration of triglycerides is
        205.0 mg/dL. \nPlugging in these values will give us 210.0 mg/dL -
        37.0 mg/dL - (205.0/5) mg/dL = 132.0 mg/dL. \nThe patients
        concentration of LDL cholestrol is 132.0 mg/dL.\n", 'Answer': 132.0}"
    """

    explanation = (
        "To compute the patient's LDL cholestrol, apply the following "
        "formula: LDL cholesterol = total cholesterol - "
        "HDL - (triglycerides / 5), where the units for total cholestrol, "
        "HDL cholestrol, and triglycerides are all mg/dL.\n"
    )

    total_cholestrol_exp, total_cholestrol = conversion_explanation(
        input_parameters["total_cholestrol"][0],
        "total cholestrol",
        386.654,
        None,
        input_parameters["total_cholestrol"][1],
        "mg/dL",
    )
    hdl_cholestrol_exp, hdl_cholestrol = conversion_explanation(
        input_parameters["hdl_cholestrol"][0],
        "hdl cholestrol",
        386.654,
        None,
        input_parameters["hdl_cholestrol"][1],
        "mg/dL",
    )
    triglycerides_exp, triglycerides = conversion_explanation(
        input_parameters["triglycerides"][0],
        "triglycerides",
        861.338,
        None,
        input_parameters["triglycerides"][1],
        "mg/dL",
    )

    explanation += total_cholestrol_exp + '\n'
    explanation += hdl_cholestrol_exp + '\n'
    explanation += triglycerides_exp + '\n'

    answer = round_number(
        total_cholestrol - hdl_cholestrol - (triglycerides / 5)
    )

    explanation += (
        f"Plugging in these values will give us {total_cholestrol} mg/dL - "
        f"{hdl_cholestrol} mg/dL - ({triglycerides}/5) mg/dL = {answer} "
        f"mg/dL.\n"
    )

    explanation += (
        f"The patients concentration of LDL cholestrol is {answer} mg/dL.\n"
    )

    return {"Explanation": explanation, "Answer": answer}


if __name__ == "__main__":
    # Defining test cases
    test_cases = [
        {
            "hdl_cholestrol": (37.0, 'mg/dL'),
            "triglycerides": (205.0, 'mg/dL'),
            "total_cholestrol": (210.0, 'mg/dL'),
        },
        {
            "total_cholestrol": (165.0, 'mg/dL'),
            "triglycerides": (104.0, 'mg/dL'),
            "hdl_cholestrol": (50.0, 'mg/dL'),
        },
    ]

    # Iterate the test cases and print the results
    for i, input_variables in enumerate(test_cases, 1):
        print(f"Test Case {i}: Input = {input_variables}")
        result = compute_ldl_explanation(input_variables)
        print(result)
        print("-" * 50)



# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
r"""
This code is borrowed and modified based on the source code from the
'MedCalc-Bench' repository.
Original repository: https://github.com/ncbi-nlp/MedCalc-Bench

Modifications include:
- rewrite function compute_anion_gap_explanation
- translation

Date: March 2025
"""

from .utils.rounding import round_number
from .utils.unit_converter_new import (
    conversion_explanation,
)


def compute_anion_gap_explanation(input_parameters):
    r"""
    Calculates the patient's anion gap and generates a detailed explanatory
    text.

    Parameters:
        input_parameters (dict): A dictionary containing the following
        key-value pairs:
            - "sodium" (array): The patient's blood sodium level in the
            format (value, unit).
                - Value (float): The blood sodium level.
                - Unit (str): The unit of blood sodium level.
            - "chloride" (array): The patient's chloride level in the
            format (value, unit).
                - Value (float): The value of chloride level.
                - Unit (str): The unit of chloride level, eg. "mmol/L",
                "mEq/L", and so on.
            - "bicarbonate" (array): The patient's bicarbonate level in the
            format (value, unit).
                - Value (float): The value of bicarbonate level.
                - Unit (str): The unit of bicarbonate level, eg. "mmol/L",
                "mEq/L", and so on.

    Returns:
        dict: Contains two key-value pairs:
            - "Explanation" (str): A detailed description of
            the calculation process.
            - "Answer" (float): The patient's anion gap.

    Notes:
        - None

    Example:
        compute_anion_gap_explanation({'chloride': [106.0, 'mEq/L'],
        'bicarbonate': [20.0, 'mEq/L'],'sodium': [140.0, 'mEq/L']})
        output: "{'Explanation': "The formula for computing a patient's
        anion gap is: sodium (mEq/L) - (chloride (mEq/L)+ bicarbonate
        (mEq/L)).\nThe concentration of sodium is 140.0 mEq/L.
        \nThe concentration of chloride is 106.0 mEq/L. \nThe concentration
        of bicarbonate is 20.0 mEq/L. \nPlugging in these values into the
        anion gap formula gives us 140.0 mEq/L - (106.0 mEq/L + 20.0 mEq/L)
        = 14.0 mEq/L. Hence, The patient's anion gap is 14.0 mEq/L.\n",
        'Answer': 14.0}"
    """

    explanation = ""
    explanation += (
        "The formula for computing a patient's anion gap is: "
        "sodium (mEq/L) - (chloride (mEq/L)+ bicarbonate (mEq/L)).\n"
    )

    sodium = input_parameters["sodium"]
    chloride = input_parameters["chloride"]
    bicarbonate = input_parameters["bicarbonate"]

    sodium_exp, sodium = conversion_explanation(
        sodium[0], "sodium", 22.99, 1, sodium[1], "mEq/L"
    )
    chloride_exp, chloride = conversion_explanation(
        chloride[0], "chloride", 35.45, 1, chloride[1], "mEq/L"
    )
    bicarbonate_exp, bicarbonate = conversion_explanation(
        bicarbonate[0], "bicarbonate", 61.02, 1, bicarbonate[1], "mEq/L"
    )

    explanation += sodium_exp + "\n"
    explanation += chloride_exp + "\n"
    explanation += bicarbonate_exp + "\n"

    answer = round_number(sodium - (chloride + bicarbonate))

    explanation += (
        f"Plugging in these values into the anion gap formula gives us "
        f"{sodium} mEq/L - ({chloride} mEq/L + "
        f"{bicarbonate} mEq/L) = {answer} mEq/L. "
    )
    explanation += f"Hence, The patient's anion gap is {answer} mEq/L.\n"

    return {"Explanation": explanation, "Answer": answer}


if __name__ == "__main__":
    # Defining test cases
    test_cases = [
        {
            "sodium": (134.0, 'mmol/L'),
            "chloride": (109.0, 'mmol/L'),
            "bicarbonate": (21.0, 'mmol/L'),
        },
        {
            'chloride': (106.0, 'mEq/L'),
            'bicarbonate': (20.0, 'mEq/L'),
            'sodium': (140.0, 'mEq/L'),
        },
    ]

    # Iterate the test cases and print the results
    for i, input_variables in enumerate(test_cases, 1):
        print(f"Test Case {i}: Input = {input_variables}")
        result = compute_anion_gap_explanation(input_variables)
        print(result)
        print("-" * 50)



# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
r"""
This code is borrowed and modified based on the source code from the
'MedCalc-Bench' repository.
Original repository: https://github.com/ncbi-nlp/MedCalc-Bench

Modifications include:
- rewrite function compute_cci_explanation
- translation

Date: March 2025
"""

from .utils.age_conversion import (
    age_conversion_explanation,
)
from .utils.unit_converter_new import (
    conversion_explanation,
)


def curb_65_explanation(input_parameters):
    r"""
    Calculates the patient's CURB-65 score and generates a
    detailed explanatory text.

        {'age': (37, 'years'),
         'sys_bp': (90.0, 'mm hg'),
         'dia_bp': (50.0, 'mm hg'),
         'respiratory_rate': (30.0, 'breaths per minute'),
         'bun': (3.5, 'mmol/L')}

    Parameters:
        input_parameters (dict): A dictionary containing the following
        key-value pairs:
            - "age" (tuple): The patient's albumin concentration in the
            format (value, unit).
                - Value (float): Age.
                - Unit (str): The unit can be "months", "years".
            - "sys_bp" (tuple): The patient's systolic blood pressure (
            value, unit).
                - Value (float): Systolic blood pressure.
                - Unit (str): The unit of systolic blood pressure,
                which can be 'mm hg'.
            - "dia_bp" (tuple): The patient's diastolic blood pressure (
            value, unit).
                - Value (float): Diastolic blood pressure.
                - Unit (str): The unit of diastolic blood pressure,
                which can be 'mm hg'.
            - "respiratory_rate" (tuple): The patient's respiratory rate
            in the format (value, unit).
                - Value (float): The value of platelet respiratory rate.
                - Unit (str): The unit of respiratory rate,
                e.g. "breaths per minute" and so on.
            - "bun" (tuple): The patient's Blood Urea Nitrogen in the
            format (value, unit).
                - Value (float): The value of BUN.
                - Unit (str): The unit of BUN,
                e.g. "mmol/L" and so on.
            - "confusion" (boolean): Whether the patient has confusion is
            not reported.

    Returns:
        dict: Contains two key-value pairs:
            - "Explanation" (str): A detailed description of
            the calculation process.
            - "Answer" (float): The patient's CURB-65 score.

    Notes:
        - None

    Example:
        curb_65_explanation({'age': (37, 'years'),
        'sys_bp': (90.0, 'mm hg'),
        'dia_bp': (50.0, 'mm hg'),
        'respiratory_rate': (30.0, 'breaths per minute'),
        'bun': (3.5, 'mmol/L')})

        output: "{'Explanation': "\n    The CURB-65 Score criteria are
        listed below:\n\n       1. Confusion: No = 0 points, Yes = +1
        point\n       2. BUN >19 mg/dL (>7 mmol/L urea): No = 0 points,
        Yes = +1 point\n       3. Respiratory Rate ≥30: No = 0 points,
        Yes = +1 point\n       4. Systolic BP <90 mmHg or Diastolic BP ≤60
        mmHg: No = 0 points, \n       Yes = +1 point\n       5. Age ≥65: No
        = 0 points, Yes = +1 point\n    \n    The total CURB-65 score is
        calculated by summing the points for each \n    criterion.\n\n\n
        The CURB-65 score is current at 0 points.\nThe patient is 37 years
        old. The patient's age is less than 65 years, and so we add 0 points
        to the score, keeping the current total at 0.\nWhether the patient
        has confusion is not reported in the note. Hence, we assume this to
        be false, and so 0 points are added to the score, making the current
        total 0.\nThe concentration of BUN is 3.5 mmol/L. We need to convert
        the concentration to mg/dL. Let's first convert the mass of BUN from
        mmol to mg. The mass of BUN is 3.5 mmol. To convert 3.5 mmol
        of BUN to mol, multiply by the conversion factor 0.001, giving
        us 3.5 mmol BUN * 0.001 mol/mmol = 0.004 mol BUN. To convert from
        mol BUN to grams, multiply by the molar mass 28.02 g/mol, which
        will give 0.004 mol BUN * 28.02 g/mol = 0.112 g BUN. To convert
        0.112 g of BUN to mg, multiply by the conversion factor 1000.0,
        giving us 0.112 g BUN * 1000.0 mg/g = 112.0 mg BUN. The current
        volume unit is L and the target volume unit is dL. The conversion
        factor is 10.0 dL for every unit of L. Our next step will be to
        divide the mass by the volume conversion factor of 10.0 to get the
        final concentration in terms of mg/dL. This will result to 112.0 mg
        BUN/10.0 dL = 11.2 mg BUN/dL. The concentration value of 3.5 mmol
        BUN/L converts to 11.2 mg BUN/dL. The patient's BUN concentration
        is less than or equal to 19 mg/dL and so 0 points are added to
        score, keeping the current total at 0.\nThe patient's respiratory
        rate is 30 breaths per minute. Because the respiratory rate is
        greater than 30 breaths per minute, 1 point is added to the score,
        making the current total 0 + 1 = 1.\nThe patient's systiolic blood
        pressure is 90 mm Hg. The patient's diastolic blood pressure is 50
        mm Hg. For a point to be added, the systiolic blood
        pressure must be less than 90 mm Hg or the diastolic blood pressure
        must be less than or equal to 60 mm Hg. Because at least one of
        these statements is true, 1 point is added to score, making the
        current total 1 + 1 = 2.\nThe patient's CURB-65 score is 2.\n",
        'Answer': 2}"
    """
    curb_65_score = 0

    explanation = r"""
    The CURB-65 Score criteria are listed below:

       1. Confusion: No = 0 points, Yes = +1 point
       2. BUN >19 mg/dL (>7 mmol/L urea): No = 0 points, Yes = +1 point
       3. Respiratory Rate ≥30: No = 0 points, Yes = +1 point
       4. Systolic BP <90 mmHg or Diastolic BP ≤60 mmHg: No = 0 points, 
       Yes = +1 point
       5. Age ≥65: No = 0 points, Yes = +1 point
    
    The total CURB-65 score is calculated by summing the points for each 
    criterion.\n\n
    """

    explanation += "The CURB-65 score is current at 0 points.\n"

    bun_exp, bun = conversion_explanation(
        input_parameters["bun"][0],
        "BUN",
        28.02,
        None,
        input_parameters["bun"][1],
        "mg/dL",
    )

    respiratory_rate = int(input_parameters["respiratory_rate"][0])
    sys_bp = int(input_parameters["sys_bp"][0])
    dia_bp = int(input_parameters["dia_bp"][0])
    age_exp, age = age_conversion_explanation(input_parameters["age"])

    explanation += age_exp

    if age >= 65:
        explanation += (
            f"The patient's age is greater than or equal to 65 "
            f"years, and so we add 1 point to the score, making "
            f"the current total {curb_65_score} + 1 = "
            f"{curb_65_score + 1}.\n"
        )
        curb_65_score += 1
    else:
        explanation += (
            f"The patient's age is less than 65 years, and so we "
            f"add 0 points to the score, keeping the current "
            f"total at {curb_65_score}.\n"
        )

    if 'confusion' not in input_parameters:
        explanation += (
            f"Whether the patient has confusion is not reported "
            f"in the note. Hence, we assume this to be false, "
            f"and so 0 points are added to the score, making the "
            f"current total {curb_65_score}.\n"
        )
    elif input_parameters["confusion"]:
        explanation += (
            f"Because the patient has confusion, "
            f"1 point is added to score making the current "
            f"total {curb_65_score} + 1 = {curb_65_score + 1}.\n"
        )
        curb_65_score += 1
    else:
        explanation += (
            f"Because the patient does not have confusion, "
            f"0 points are added to the score, keeping the score "
            f"at {curb_65_score}.\n"
        )

    explanation += bun_exp

    if bun > 19:
        explanation += (
            f"The patient's BUN concentration is greater than 19 "
            f"mg/dL and so we add 1 point to score making the "
            f"current total {curb_65_score} + 1 = "
            f"{curb_65_score + 1}.\n"
        )
        curb_65_score += 1
    else:
        explanation += (
            f"The patient's BUN concentration is less than or "
            f"equal to 19 mg/dL and so 0 points are added to "
            f"score, keeping the current total at "
            f"{curb_65_score}.\n"
        )

    explanation += (
        f"The patient's respiratory rate is {respiratory_rate} "
        f"breaths per minute. "
    )

    if respiratory_rate >= 30:
        explanation += (
            f"Because the respiratory rate is greater than 30 "
            f"breaths per minute, 1 point is added to the score, "
            f"making the current total {curb_65_score} + 1 = "
            f"{curb_65_score + 1}.\n"
        )
        curb_65_score += 1
    else:
        explanation += (
            f"Because the respiratory rate is greater than 30 "
            f"breaths per minute, 0 points are added to the "
            f"score, keeping the current total at "
            f"{curb_65_score}.\n"
        )

    explanation += (
        f"The patient's systiolic blood pressure is {sys_bp} mm "
        f"Hg. The patient's diastolic blood pressure is {dia_bp} "
        f"mm Hg. "
    )

    if sys_bp < 90 or dia_bp <= 60:
        explanation += (
            f"For a point to be added, the systiolic "
            f"blood pressure must be less than 90 mm Hg or the "
            f"diastolic blood pressure must be less than or "
            f"equal to 60 mm Hg. Because at least one of these "
            f"statements is true, 1 point is added to score, "
            f"making the current total {curb_65_score} + 1 = "
            f"{curb_65_score + 1}.\n"
        )
        curb_65_score += 1
    else:
        explanation += (
            f"For a point to be added, the systiolic "
            f"blood pressure must be less than 90 mm Hg or the "
            f"diastolic blood pressure must be less than or "
            f"equal to 60 mm Hg. Because neither of these "
            f"statements are true, 0 points are added to score, "
            f"keeping the current total to {curb_65_score}.\n"
        )

    explanation += f"The patient's CURB-65 score is {curb_65_score}.\n"

    return {"Explanation": explanation, "Answer": curb_65_score}


if __name__ == "__main__":
    # Defining test cases
    test_cases = [
        {
            'age': (37, 'years'),
            'sys_bp': (90.0, 'mm hg'),
            'dia_bp': (50.0, 'mm hg'),
            'respiratory_rate': (30.0, 'breaths per minute'),
            'bun': (3.5, 'mmol/L'),
        }
    ]

    # Iterate the test cases and print the results
    for i, input_variables in enumerate(test_cases, 1):
        print(f"Test Case {i}: Input = {input_variables}")
        result = curb_65_explanation(input_variables)
        print(result)
        print("-" * 50)



# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
r"""
This code is borrowed and modified based on the source code from
    the 'MedCalc-Bench' repository.
Original repository: https://github.com/ncbi-nlp/MedCalc-Bench

Modifications include:
- None

Date: March 2025
"""


def age_conversion(input):
    count = 0
    while count < len(input):
        if "year" in input[count + 1]:
            return input[count]
        elif "months" in input[count + 1]:
            return input[count] // 12
        elif "weeks" in input[count + 1]:
            return input[count] // 52
        elif "days" in input[count + 1]:
            return 0
        count += 2


def age_conversion_explanation(input):
    count = 0
    text = "The patient is "

    if len(input) == 2 and input[1] == "months":
        if input[0] // 12 >= 1:
            years = input[0] // 12
            months = input[0] % 12

            add_s = "s"
            if years == 1:
                add_s = ""

            return (
                f"This means that the patient is {years} year{add_s} and "
                f"{months} old",
                input[0] // 12,
            )

    while count < len(input):
        text += f"{input[count]} {input[count + 1]}"
        if len(input) - count - 2 == 0:
            text += " old. "
        elif len(input) - count - 2 > 2:
            text += ", "
        elif len(input) - count - 2 == 2:
            text += ", and "
        count += 2

    if "year" not in text:
        text += "This means the patient is 0 years old.\n"

    return text, age_conversion(input)



# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
r"""
This code is borrowed and modified based on the source code from the
'MedCalc-Bench' repository.
Original repository: https://github.com/ncbi-nlp/MedCalc-Bench

Modifications include:
- rewrite function compute_steroid_conversion_explanation
- translation

Date: March 2025
"""

from .utils.rounding import round_number
from .utils.unit_converter_new import (
    conversion_explanation,
)


def compute_steroid_conversion_explanation(input_parameters):
    r"""
    Calculates the patient's equivalent dosage of MethylPrednisoLONE IV and
    generates a detailed explanatory text.

    Parameters:
        input_parameters (dict): A dictionary containing the following
        key-value pairs:
            - "input steroid" (tuple): The patient's blood sodium level in
            the format (value, unit).
                - Steroid name (str): The various corticosteroids are below:
                    - Betamethasone: Route = IV, Equivalent Dose = 0.75 mg
                    - Cortisone: Route = PO, Equivalent Dose = 25 mg
                    - Dexamethasone (Decadron): Route = IV or PO,
                    Equivalent Dose = 0.75 mg
                    - Hydrocortisone: Route = IV or PO, Equivalent Dose = 20 mg
                    - MethylPrednisoLONE: Route = IV or PO,
                    Equivalent Dose = 4 mg
                    - PrednisoLONE: Route = PO, Equivalent Dose = 5 mg
                    - PredniSONE: Route = PO, Equivalent Dose = 5 mg
                    - Triamcinolone: Route = IV, Equivalent Dose = 4 mg
                - Value (float): The blood sodium level.
                - Unit (str): The unit of blood sodium level.
            - "target steroid" (str): The target steroid (str).
                - "Betamethasone IV"
                - "Cortisone PO"
                - "Dexamethasone IV"
                - "Dexamethasone PO"
                - "Hydrocortisone IV"
                - "Hydrocortisone PO"
                - "MethylPrednisoLONE IV"
                - "MethylPrednisoLONE PO"
                - "PrednisoLONE PO"
                - "PredniSONE PO"
                - "Triamcinolone IV"

    Returns:
        dict: Contains two key-value pairs:
            - "Explanation" (str): A detailed description of
            the calculation process.
            - "Answer" (float): The patient's equivalent dosage of
            MethylPrednisoLONE IV.

    Notes:
        - None

    Example:
        compute_steroid_conversion_explanation({
            'input steroid': ['Hydrocortisone PO', 190.936, 'mg'],
            'target steroid': 'MethylPrednisoLONE IV'})

        output: "{'Explanation': '\n The Steroid Conversions providing
        equivalent doses for various corticosteroids are listed below:\n 1.
        Betamethasone: Route = IV, Equivalent Dose = 0.75 mg\n 2.
        Cortisone: Route = PO, Equivalent Dose = 25 mg\n 3. Dexamethasone
        (Decadron): Route = IV or PO, Equivalent Dose = 0.75 mg\n 4.
        Hydrocortisone: Route = IV or PO, Equivalent Dose = 20 mg\n 5.
        MethylPrednisoLONE: Route = IV or PO, Equivalent Dose = 4 mg\n 6.
        PrednisoLONE: Route = PO, Equivalent Dose = 5 mg\n 7. PredniSONE:
        Route = PO, Equivalent Dose = 5 mg\n 8. Triamcinolone: Route = IV,
        Equivalent Dose = 4 mg \n \n\nThe mass of Hydrocortisone PO is
        190.936 mg. To convert from the Hydrocortisone PO to
        MethylPrednisoLONE IV, multiply by the conversion factor, 0.2 mg
        MethylPrednisoLONE IV/Hydrocortisone PO, giving us 190.936 mg
        Hydrocortisone PO * 0.2 mg MethylPrednisoLONE IV/mg Hydrocortisone
        PO = 38.187 mg MethylPrednisoLONE IV. 190.936 mg of Hydrocortisone
        PO is equal to 38.187 mg of MethylPrednisoLONE IV.\n', 'Answer':
        38.187}"
    """

    explanation = """
        The Steroid Conversions providing equivalent doses for various
        corticosteroids are listed below:
            1. Betamethasone: Route = IV, Equivalent Dose = 0.75 mg
            2. Cortisone: Route = PO, Equivalent Dose = 25 mg
            3. Dexamethasone (Decadron): Route = IV or PO, Equivalent Dose
                = 0.75 mg
            4. Hydrocortisone: Route = IV or PO, Equivalent Dose = 20 mg
            5. MethylPrednisoLONE: Route = IV or PO, Equivalent Dose = 4 mg
            6. PrednisoLONE: Route = PO, Equivalent Dose = 5 mg
            7. PredniSONE: Route = PO, Equivalent Dose = 5 mg
            8. Triamcinolone: Route = IV, Equivalent Dose = 4 mg
        """
    conversion_dict = {
        "betamethasone iv": 1,
        "cortisone po": 33.33,
        "dexamethasone iv": 1,
        "dexamethasone po": 1,
        "hydrocortisone iv": 26.67,
        "hydrocortisone po": 26.67,
        "methylprednisolone iv": 5.33,
        "methylprednisolone po": 5.33,
        "prednisolone po": 6.67,
        "prednisone po": 6.67,
        "triamcinolone iv": 5.33,
    }

    explanation += "\n\n"
    input_drug_mass_exp, input_drug_mass = conversion_explanation(
        input_parameters["input steroid"][1],
        input_parameters["input steroid"][0],
        None,
        None,
        input_parameters["input steroid"][2],
        "mg",
    )
    explanation += input_drug_mass_exp

    target_drug_name = input_parameters["target steroid"].lower()
    input_drug_name = input_parameters["input steroid"][0].lower()
    input_unit = input_parameters["input steroid"][2]

    from_multiplier = conversion_dict[input_drug_name]
    to_multiplier = conversion_dict[target_drug_name]

    conversion_factor = round_number(to_multiplier / from_multiplier)
    converted_amount = round_number(input_drug_mass * conversion_factor)
    input_drug_mass = round_number(input_drug_mass)

    explanation += (
        f"To convert from the {input_drug_name} to {target_drug_name}, "
        f"multiply by the conversion factor, "
        f"{conversion_factor} mg {target_drug_name}/{input_drug_name}, "
        f"giving us {input_drug_mass} mg {input_drug_name} * "
        f"{conversion_factor} mg "
        f"{target_drug_name}/mg {input_drug_name} = {converted_amount} mg "
        f"{target_drug_name}. "
    )

    explanation += (
        f"{input_drug_mass} {input_unit} of {input_drug_name} is equal to "
        f"{converted_amount} mg of {target_drug_name}.\n"
    )

    return {"Explanation": explanation, "Answer": converted_amount}


if __name__ == "__main__":
    # Defining test cases
    test_cases = [
        {
            'input steroid': ['Hydrocortisone PO', 190.936, 'mg'],
            'target steroid': 'MethylPrednisoLONE IV',
        },
        {
            'input steroid': ['Dexamethasone PO', 8.58, 'mg'],
            'target steroid': 'Dexamethasone IV',
        },
    ]

    # Iterate the test cases and print the results
    for i, input_variables in enumerate(test_cases, 1):
        print(f"Test Case {i}: Input = {input_variables}")
        result = compute_steroid_conversion_explanation(input_variables)
        print(result)
        print("-" * 50)



# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ========= Copyright 2023-2024 @ CAMEL-AI.org. All Rights Reserved. =========
r"""
This code is borrowed and modified based on the source code from the
'MedCalc-Bench' repository.
Original repository: https://github.com/ncbi-nlp/MedCalc-Bench

Modifications include:
- rewrite function compute_sodium_correction_hyperglycemia_explanation
- translation

Date: March 2025
"""

from .utils.rounding import round_number
from .utils.unit_converter_new import (
    conversion_explanation,
)


def compute_sodium_correction_hyperglycemia_explanation(input_variables):
    r"""
    Calculates the patient's corrected sodium concentration in terms of
    mEq/L and generates a detailed explanatory text.

    Parameters:
        input_variables (dict): A dictionary containing the following
        key-value pairs:
            - "glucose" (tuple): The patient's blood glucose level in the
            format (value, unit).
                - Value (float): The value of blood glucose level.
                - Unit (str): The unit of blood glucose level,
                eg. "mmol/L", "mEq/L", and so on.
            - "sodium" (array): The patient's blood sodium level in the
            format (value, unit).
                - Value (float): The blood sodium level.
                - Unit (str): The unit of blood sodium level.

    Returns:
        dict: Contains two key-value pairs:
            - "Explanation" (str): A detailed description of
            the calculation process.
            - "Answer" (float): The patient's corrected sodium concentration.

    Notes:
        - None

    Example:
        free_water_deficit_explanation({
        'glucose': (90.0, 'mg/dL'),
        'sodium': (134.0, 'mEq/L')})

        output: "{'Explanation': "The formula for Sodium Correction for
        Hyperglycemia is Measured sodium + 0.024 * (Serum glucose - 100),
        where Measured Sodium is the sodium concentration in mEq/L and the
        Serum glucose is the concentration of glucose in mg/dL.\nThe
        concentration of sodium is 134.0 mEq/L. \nThe concentration of
        glucose is 90.0 mg/dL. \nPlugging in these values into the formula
        gives us 134.0 mEq/L + 0.024 * (90.0 - 100) = 133.76 mEq/L.\nHence,
        the patient's corrected concentration of sodium is 133.76 mEq/L.\n",
        'Answer': 133.76}"
    """

    sodium = input_variables["sodium"]
    glucose = input_variables["glucose"]

    sodium_explanation, sodium = conversion_explanation(
        sodium[0], "sodium", 22.99, 1, sodium[1], "mEq/L"
    )
    glucose_explanation, glucose = conversion_explanation(
        glucose[0], "glucose", 180.16, None, glucose[1], "mg/dL"
    )

    corrected_sodium = round_number(sodium + 0.024 * (glucose - 100))

    explanation = (
        "The formula for Sodium Correction for Hyperglycemia is "
        "Measured sodium + 0.024 * (Serum glucose - 100), "
        "where Measured Sodium is the sodium concentration in "
        "mEq/L and the Serum glucose is the concentration of "
        "glucose in mg/dL.\n"
    )

    explanation += sodium_explanation + "\n"
    explanation += glucose_explanation + "\n"

    explanation += (
        f"Plugging in these values into the formula gives us "
        f"{sodium} mEq/L + 0.024 * ({glucose} - 100) = "
        f"{corrected_sodium} mEq/L.\n"
    )

    explanation += (
        f"Hence, the patient's corrected concentration of sodium "
        f"is {corrected_sodium} mEq/L.\n"
    )

    return {"Explanation": explanation, "Answer": corrected_sodium}


if __name__ == "__main__":
    # Defining test cases
    test_cases = [
        {'glucose': (90.0, 'mg/dL'), 'sodium': (134.0, 'mEq/L')},
    ]

    # Iterate the test cases and print the results
    for i, input_variables in enumerate(test_cases, 1):
        print(f"Test Case {i}: Input = {input_variables}")
        result = compute_sodium_correction_hyperglycemia_explanation(
            input_variables
        )
        print(result)
        print("-" * 50)
